/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// Standard imports
import java.awt.*;
import javax.swing.*;
import java.beans.*;

//
public class InstructPanel extends JPanel {
    private GridBagLayout layoutMain = null;
    private JPanel        panelHelp = null;
    private JPanel        panelOptions = null;
    private GridBagLayout layoutHelp = null;
    private JLabel        labelTitle = null;
    private BorderLayout  layoutOptions = null;
    private JTextArea     textInstruct = null;

    public InstructPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    //
    protected String getInstruction() {
        return textInstruct.getText();
    }

    protected void setInstructions(String text) {
        textInstruct.setText(text);
    }

    protected void setTitle(String text) {
        labelTitle.setText(text);
    }

    protected String getTitle() {
        return labelTitle.getText();
    }

    protected void addInstuctor(Instructor i) {
        if (i instanceof JPanel) {
            panelOptions.removeAll();
            panelOptions.add((JPanel) i, BorderLayout.CENTER);
            setInstructions(i.getInstructions());
            setTitle(i.getTitle());
        } else {
            throw new RuntimeException("Instructor not a JPanel "
                                       + i.getTitle());
        }
    }

    //
    private void pmInit() {
        Font font = labelTitle.getFont();

        font = new Font(font.getName(), Font.BOLD, font.getSize());
        labelTitle.setFont(font);
        font = new Font(font.getName(), 0, font.getSize());
        textInstruct.setFont(font);
        textInstruct.setBackground(labelTitle.getBackground());
    }

    private void jbInit() throws Exception {
        labelTitle = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                JLabel.class.getName());
        panelHelp = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelOptions = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutHelp =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutOptions =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        textInstruct =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());
        labelTitle.setText("TITLE");
        panelHelp.setLayout(layoutHelp);
        textInstruct.setWrapStyleWord(true);
        textInstruct.setLineWrap(true);
        textInstruct.setEnabled(false);
        textInstruct.setText("INSTRUCTIONS");
        textInstruct.setDisabledTextColor(UIManager.getColor("textText"));
        textInstruct.setBackground(UIManager.getColor("text"));
        textInstruct.setEditable(false);
        panelHelp.add(labelTitle,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(3, 5, 1, 5), 0, 0));
        panelHelp.add(textInstruct,
                      new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(1, 5, 2, 5), 0, 0));
        panelOptions.setLayout(layoutOptions);
        this.setLayout(layoutMain);
        this.add(panelHelp,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.NORTH,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(panelOptions,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.9,
                                        GridBagConstraints.SOUTH,
                                        GridBagConstraints.BOTH,
                                        new Insets(2, 5, 2, 5), 0, 0));
    }

}
