/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.properties;

// ToolBox imports
import org.enhydra.tool.common.DialogPanel;
import org.enhydra.tool.common.SwingUtil;

// Kelp imports
import org.enhydra.kelp.common.PropUtil;

// Standard imports
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.util.ResourceBundle;

//
public class XMLCParameterViewer extends DialogPanel {

    //
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private DefaultListModel    listModel = null;
    private LocalListListener   listListener = null;
    private LocalButtonListener buttonListener = null;
    private BorderLayout        layoutRoot = null;
    private GridBagLayout       layoutTab = null;
    private GridBagLayout       layoutEdit = null;
    private GridBagLayout       layoutControl = null;
    private JTabbedPane         tab = null;
    private JPanel              panelTab = null;
    private JPanel              panelEdit = null;
    private JPanel              panelControl = null;
    private JButton             buttonAdd = null;
    private JButton             buttonEdit = null;
    private JButton             buttonRemove = null;
    private JButton             buttonUp = null;
    private JButton             buttonDown = null;
    private JButton             buttonOK = null;
    private JButton             buttonCancel = null;
    private JScrollPane         scroll = null;
    private JList               list = null;

    public static void main(String[] args) {
        SwingUtil.setLookAndFeelToSystem();
        XMLCParameterViewer viewer = null;

        viewer = new XMLCParameterViewer();
        viewer.showDialog();
        System.out.println(viewer.getParameters());
        System.exit(0);
    }

    public XMLCParameterViewer() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public String getParameters() {
        String   out = new String();
        String[] params = new String[0];

        params = new String[listModel.getSize()];
        for (int i = 0; i < params.length; i++) {
            params[i] = listModel.getElementAt(i).toString();
        }
        params = PropUtil.cleanXMLCParameters(params);
        out = PropUtil.XMLCParametersToString(params);
        return out;
    }

    public void setParameters(String paramLine) {
        String[] params = new String[0];

        params = PropUtil.XMLCParametersToArray(paramLine);
        params = PropUtil.cleanXMLCParameters(params);
        listModel.removeAllElements();
        for (int i = 0; i < params.length; i++) {
            listModel.addElement(params[i]);
        }
        list.setModel(listModel);
        list.updateUI();
    }

    //
    //
    protected void clearDialog() {
        super.clearDialog();
    }

    //
    //
    private void refreshEditButtons() {
        int index = list.getSelectedIndex();

        buttonRemove.setEnabled(index > -1);
        buttonEdit.setEnabled(index > -1);
        buttonUp.setEnabled(index > 0);
        buttonDown.setEnabled((index > -1)
                              && (index < (listModel.getSize() - 1)));
    }

    private void removeSelection() {
        int index = list.getSelectedIndex();

        if (index > -1) {
            listModel.removeElementAt(index);
        }
    }

    private void moveSelection(int direction) {
        int index = list.getSelectedIndex();

        if (index > -1) {
            String selection = null;

            selection = listModel.getElementAt(index).toString();
            listModel.removeElementAt(index);
            listModel.insertElementAt(selection, index + direction);
        }
    }

    private void add() {
        XMLCParameterEditor editor = null;
        String              selection = new String();

        editor = new XMLCParameterEditor();
        if (showEditor(editor) == XMLCParameterEditor.OK_OPTION) {
            selection = editor.getParameter();
            if (selection.trim().length() > 0) {
                listModel.addElement(selection);
                list.setSelectedIndex(listModel.getSize() - 1);
            }
        }
    }

    private void editSelection() {
        XMLCParameterEditor editor = null;
        int                 index = list.getSelectedIndex();

        if (index > -1) {
            String selection = null;

            selection = listModel.getElementAt(index).toString();
            editor = new XMLCParameterEditor();
            editor.setParameter(selection);
            if (showEditor(editor) == XMLCParameterEditor.OK_OPTION) {
                selection = editor.getParameter();
                listModel.setElementAt(selection, index);
            }
        }
    }

    private int showEditor(XMLCParameterEditor editor) {
        int option = XMLCParameterEditor.CANCEL_OPTION;

        editor.setOwner((Window) getTopLevelAncestor());
        editor.setModal(true);
        return editor.showDialog();
    }

    private void pmInit() {
        listModel = new DefaultListModel();
        listListener = new LocalListListener();
        buttonListener = new LocalButtonListener();
        list.addListSelectionListener(listListener);
        list.requestDefaultFocus();
        list.setModel(listModel);
        list.setCellRenderer(new ToolTipCellRenderer());
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        buttonAdd.addActionListener(buttonListener);
        buttonEdit.addActionListener(buttonListener);
        buttonRemove.addActionListener(buttonListener);
        buttonUp.addActionListener(buttonListener);
        buttonDown.addActionListener(buttonListener);
        buttonOK.addActionListener(buttonListener);
        buttonCancel.addActionListener(buttonListener);
        refreshEditButtons();
        setTitle(res.getString("XMLC_Parameters"));
        setModal(true);
    }

    private void jbInit() throws Exception {
        layoutRoot =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        layoutTab =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutEdit =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutControl =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        panelTab = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        panelEdit = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelControl = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonEdit = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonUp = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                               JButton.class.getName());
        buttonDown = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        list = (JList) Beans.instantiate(getClass().getClassLoader(),
                                         JList.class.getName());
        buttonOK = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                               JButton.class.getName());
        buttonCancel =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonAdd.setPreferredSize(new Dimension(100, 27));
        buttonAdd.setMnemonic('A');
        buttonAdd.setText(res.getString("buttonAdd_Text"));
        buttonEdit.setPreferredSize(new Dimension(100, 27));
        buttonEdit.setMnemonic('E');
        buttonEdit.setText(res.getString("Edit"));
        buttonRemove.setPreferredSize(new Dimension(100, 27));
        buttonRemove.setMnemonic('R');
        buttonRemove.setText(res.getString("Remove"));
        buttonUp.setPreferredSize(new Dimension(100, 27));
        buttonUp.setMnemonic('U');
        buttonUp.setText(res.getString("buttonUp_Text"));
        buttonDown.setPreferredSize(new Dimension(100, 27));
        buttonDown.setMnemonic('D');
        buttonDown.setText(res.getString("buttonDown_Text"));
        buttonOK.setPreferredSize(new Dimension(100, 27));
        buttonOK.setText(res.getString("OK"));
        buttonOK.setDefaultCapable(true);
        buttonCancel.setPreferredSize(new Dimension(100, 27));
        buttonCancel.setText(res.getString("Cancel"));
        scroll = new JScrollPane(list);
        panelEdit.setLayout(layoutEdit);
        tab.setPreferredSize(new Dimension(400, 300));
        panelEdit.add(buttonAdd,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 5), 0, 0));
        panelEdit.add(buttonEdit,
                      new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 5), 0, 0));
        panelEdit.add(buttonRemove,
                      new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 20, 5), 0, 0));
        panelEdit.add(buttonUp,
                      new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(20, 5, 5, 5), 0, 0));
        panelEdit.add(buttonDown,
                      new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 5), 0, 0));
        panelTab.setLayout(layoutTab);
        panelTab.add(scroll,
                     new GridBagConstraints(0, 0, 1, 1, 0.3, 0.1,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.BOTH,
                                            new Insets(0, 0, 0, 0), 0, 0));
        panelTab.add(panelEdit,
                     new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.BOTH,
                                            new Insets(5, 5, 5, 5), 0, 0));
        tab.add(panelTab, res.getString("Parameters"));
        panelControl.setLayout(layoutControl);
        panelControl.add(buttonOK,
                         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        panelControl.add(buttonCancel,
                         new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        this.setLayout(layoutRoot);
        this.add(tab, BorderLayout.NORTH);
        this.add(panelControl, BorderLayout.SOUTH);
    }

    //
    private class ToolTipCellRenderer extends DefaultListCellRenderer {
        public Component getListCellRendererComponent(JList list,
                Object value, int index, boolean isSelected,
                boolean cellHasFocus) {
            Component c = null;

            c = super.getListCellRendererComponent(list, value, index,
                                                   isSelected, cellHasFocus);
            if (c instanceof JComponent) {
                JComponent jc = (JComponent) c;

                jc.setToolTipText(value.toString());
            }
            return c;
        }

    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonAdd) {
                add();
            } else if (source == buttonEdit) {
                editSelection();
            } else if (source == buttonRemove) {
                removeSelection();
                refreshEditButtons();
            } else if (source == buttonUp) {
                moveSelection(-1);
                refreshEditButtons();
            } else if (source == buttonDown) {
                moveSelection(1);
                refreshEditButtons();
            } else if (source == buttonOK) {
                setOption(DialogPanel.OK_OPTION);
                clearDialog();
            } else if (source == buttonCancel) {
                setOption(DialogPanel.CANCEL_OPTION);
                clearDialog();
            }
        }

    }

    //
    private class LocalListListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent event) {
            if (event.getValueIsAdjusting()) {

                // do nothing
            } else {
                refreshEditButtons();
            }
        }

    }
}
