/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.xmlc;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.SwingUtil;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.swing.AddinInnerPanel;
import org.enhydra.kelp.common.swing.FileNodeSelectionPanel;

// JDK
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;
import java.awt.*;
import java.beans.*;
import javax.swing.*;

//
public class XMLCInnerPanel extends AddinInnerPanel {

    //
    static ResourceBundle          res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    //
    private FileNodeSelectionPanel selectionPanel = null;
    private boolean                standalone = false;
    private JPanel                 panelOptionFiller0 = null;
    private JPanel                 panelOptionFiller1 = null;
    private JPanel                 panelOptionFiller2 = null;
    private GridBagLayout          layoutMain = null;
    private JPanel                 propertyPanelHolder = null;
    private AddinInnerPanel        propertyPanel = null;
    private XMLCOutputPanel        outputPanel = null;
    private JTabbedPane            tab = null;
    private WriteListener[]        writeListeners = new WriteListener[0];
    private String                 progressTitle = new String();

    public static XMLCInnerPanel XMLCfindAncestor(Component comp) {
        Component      found = null;
        XMLCInnerPanel ancestor = null;

        found = AddinInnerPanel.findAncestor(comp);
        if (found == null) {}
        else if (found instanceof XMLCInnerPanel) {
            ancestor = (XMLCInnerPanel) found;
        }
        return ancestor;
    }

    public XMLCInnerPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // overwrite org.enhydra.kelp.common.swing.InnerPanel
    public void clearAll() {
        super.clearAll();
        layoutMain = null;
        writeListeners = new WriteListener[0];
    }

    public FileNodeSelectionPanel getSelectionPanel() {
        return selectionPanel;
    }

    public AddinInnerPanel getPropertyPanel() {
        return propertyPanel;
    }

    public void selectOutputTab() {
        selectTab(outputPanel);
    }

    public void selectOptionTab() {
        selectTab(panelOptionFiller0);
    }

    //
    // PROTECTED
    //
    protected void setPropertyPanel(AddinInnerPanel p) {
        if (propertyPanel == null) {
            propertyPanelHolder.removeAll();
        } else {
            propertyPanelHolder.remove(propertyPanel);
        }
        propertyPanel = p;
        propertyPanelHolder.add(propertyPanel, BorderLayout.CENTER);
    }

    protected void setSelectText(String[] s) {
        getSelectionPanel().setSelectText(s);
    }

    protected String[] getSelectText() {
        return getSelectionPanel().getSelectText();
    }

    protected synchronized WriteListener[] getWriteListeners() {
        return writeListeners;
    }

    protected synchronized void addWriteListener(WriteListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(writeListeners));
        if (!list.contains(l)) {
            list.add(l);
            list.trimToSize();
            writeListeners = new WriteListener[list.size()];
            writeListeners = (WriteListener[]) list.toArray(writeListeners);
        }
        list.clear();
    }

    protected synchronized void removeWriteListener(WriteListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(writeListeners));
        if (list.contains(l)) {
            list.remove(l);
            list.trimToSize();
            writeListeners = new WriteListener[list.size()];
            writeListeners = (WriteListener[]) list.toArray(writeListeners);
        }
        list.clear();
    }

    /**
     * Get source files that can be selected for compilation.
     */
    protected OtterFileNode[] getNodes() {
        return selectionPanel.getNodes();
    }

    /**
     * Set the source files that can be selected for compilation
     */
    protected void setNodes(OtterFileNode[] n) {
        selectionPanel.setNodes(n);
    }

    // override AddinInnerPanel
    public void read(OtterNode node) {
        super.read(node);
        propertyPanel.read(node);
        outputPanel.setProject(getProject());
    }

    // override AddinInnerPanel
    public void write(OtterNode node) throws DataValidationException {
        super.write(node);
        if (getProject() == null) {
            System.err.println("XMLCInnerPanel.write(OtterProject p) : no project set");
        } else {
            propertyPanel.write(node);
            outputPanel.setProject(getProject());
        }
    }

    // override AddinInnerPanel
    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[1];

        comps[0] = tab;
        return comps;
    }

    //
    // PRIVATE
    //
    // override AddinInnerPanel
    public void save() {
        super.save();
        propertyPanel.save();
    }

    protected void clearOutput() {
        outputPanel.clearOutput();
    }

    //
    // PRIVATE
    //
    private void selectTab(final JPanel goPanel) {
        SelectTab runSwing = null;

        runSwing = new SelectTab(tab, goPanel);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                runSwing.run();
            } else {
                SwingUtilities.invokeAndWait(runSwing);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {

            //
        }
    }

    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
        tab.add(res.getString("Selections"), selectionPanel);
        tab.add(res.getString("Options"), panelOptionFiller0);
        tab.add(res.getString("Output"), outputPanel);
        addWriteListener(outputPanel);
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelOptionFiller0 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        panelOptionFiller1 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        panelOptionFiller2 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        selectionPanel =
            (FileNodeSelectionPanel) Beans.instantiate(getClass().getClassLoader(),
                FileNodeSelectionPanel.class.getName());
        propertyPanelHolder =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        outputPanel =
            (XMLCOutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                                XMLCOutputPanel.class.getName());
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        panelOptionFiller0.setLayout(new GridBagLayout());
        panelOptionFiller0.add(panelOptionFiller1,
                               new GridBagConstraints(0, 0, 1, 1, 0.15, 0.15,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.BOTH,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.add(propertyPanelHolder,
                               new GridBagConstraints(1, 0, 1, 4, 0.7, 0.7,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.NONE,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.add(panelOptionFiller2,
                               new GridBagConstraints(2, 0, 1, 1, 0.15, 0.15,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.BOTH,
                               new Insets(0, 0, 0, 0), 0, 0));
        this.setLayout(layoutMain);
        this.add(tab,
                 new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 5, 5));
    }

    //
    private class SelectTab implements Runnable {
        private JTabbedPane tp = null;
        private JPanel      go = null;

        protected SelectTab(JTabbedPane t, JPanel g) {
            tp = t;
            go = g;
        }

        synchronized public void run() {
            tp.setSelectedComponent(go);
            try {
                SwingUtilities.windowForComponent(tp).repaint();
            } catch (NullPointerException e) {

                // ignore for windows not yet open.
            }
        }

    }
}
