/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.actions.OpenAction;
import org.openide.actions.ViewAction;
import org.openide.filesystems.FileSystemCapability;
import org.openide.filesystems.FileStateInvalidException;

import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.forte.node.ForteDocumentNode;
import org.enhydra.kelp.common.properties.XMLCNodePropertyPanel;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.forte.XMLCSettings;

import java.lang.IllegalAccessException;
import java.lang.reflect.InvocationTargetException;
import java.util.ResourceBundle;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.awt.Component;
import java.awt.Image;
import javax.swing.ImageIcon;

/** A node to represent this object.
 *
 * @author  rees0234
 */
public class XMLCDataNode extends DataNode implements PropertyKeys
{
    public static final String PROP_SHEET_NAME = "xmlc.sheet";
    protected Sheet.Set propSet = null;
    private static ResourceBundle  bundle = NbBundle.getBundle (XMLCDataNode.class);
    private static final String XMLC_ICON_BASE = "/org/enhydra/kelp/forte/enhydraObject";


    public XMLCDataNode (XMLCDataObject obj)
    {
        this (obj, Children.LEAF);
    }

    public XMLCDataNode (XMLCDataObject obj, Children ch)
    {
        super (obj, ch);
        setIconBase (XMLC_ICON_BASE);
    }

    protected XMLCDataObject getXMLCDataObject ()
    {
        return (XMLCDataObject) getDataObject ();
    }

    public boolean hasCustomizer()
    {
        return true;
    }

    public Component getCustomizer()
    {
        XMLCNodePropertyPanel panel = new XMLCNodePropertyPanel();
        panel.setSaveOnClose(true);
        panel.read(new OtterXMLCNode(new ForteDocumentNode(getXMLCDataObject())));
        return panel;
    }

    protected Sheet createSheet () {

        Sheet s = Sheet.createDefault ();
        propSet = s.get (Sheet.PROPERTIES);

	if (propSet == null) {
	    propSet = new Sheet.Set ();
	    propSet.setName (PROP_SHEET_NAME);
            propSet.setDisplayName (bundle.getString("LBL_XMLC_sheet"));
            propSet.setShortDescription (bundle.getString("HINT_XMLC_sheet"));
	}

        PropertySupport p = new BooleanProp(NAME_SELECTED,bundle.getString("LBL_Selected"), bundle.getString("HINT_Selected"));
        propSet.put (p);

	return s;
    }

    public SystemAction getDefaultAction () {
        return SystemAction.get (ViewAction.class);
    }

        /** A property for a boolean property and its value. */
    private class BooleanProp extends PropertySupport.ReadWrite
    {
        /** The property name. */
        private String property;
        /** Make a property.
         * @param property the property name to use
         */
        public BooleanProp (String property, String disp, String desc)
        {
            super (property, String.class, disp, desc);
            this.property = property;
        }

        public Object getValue ()
        {
            return getXMLCDataObject().getProperty(property);
        }

        public void setValue (Object nue) {
            getXMLCDataObject().setProperty(property, (String)nue);
        }

        public PropertyEditor getPropertyEditor()
        {
            return new BoolEd();
        }
    }

    public static class BoolEd extends PropertyEditorSupport {

        private static final String[] tags = {
            NbBundle.getMessage (XMLCSettings.class, "LBL_true"),
            NbBundle.getMessage (XMLCSettings.class, "LBL_false")
	};

        public String[] getTags () {
            return tags;
        }

        public String getAsText () {
             return tags[((String)getValue()).equals(tags[0]) ? 0 : 1];
        }

        public void setAsText (String text) throws IllegalArgumentException {
           setValue(text);
        }

    }
}
