/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Devel doper of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.tool;

import java.io.File;
// JBuilder imports
import com.borland.jbuilder.JBuilderHelp;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.help.ZipHelpBook;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserAction;
import com.borland.primetime.vfs.Filesystem;
import com.borland.primetime.vfs.VFS;

import com.borland.primetime.node.FileType;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.ide.ContextActionProvider;
import com.borland.primetime.node.Node;
import com.borland.primetime.ide.ContentManager;
import javax.swing.Action;
import com.borland.primetime.ide.BrowserIcons;
import com.borland.primetime.wizard.Wizard;
import org.enhydra.kelp.jbuilder.wizard.Enhydra3AppWizard;
import com.borland.primetime.ide.ProjectView;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.swing.KelpIconSet;
import org.enhydra.kelp.common.dods.CoreDodsTool;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

import com.borland.jbuilder.paths.PathSet;
import com.borland.jbuilder.paths.PathSetManager;
import com.borland.primetime.vfs.Url;
import com.borland.jbuilder.paths.JDKPathSet;
import com.borland.primetime.node.Project;
import com.borland.jbuilder.node.PackageNode;
import com.borland.primetime.node.FileNode;

import org.enhydra.dods.generator.DODSGenerator;

//
public class DodsTool extends AbstractBasicTool {


    //
    private final String HELP_LINK = KelpInfo.getAddinHelpURL();    // nores

    //
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        AbstractBasicTool.initAllTools(majorVersion, minorVersion);
            //Dusan 22.01.03.
            FileType ft = FileType.getFileType("doml");
            if (ft == null) {
              ft.registerFileType("doml", FileType.getFileType("xml"));
            }

            //Dusan 22.01.03.
            //Setting right click on Node tab in ContentManager
            ContentManager.registerContextActionProvider(new ContextActionProvider() {
              public Action getContextAction(Browser browser, Node[] nodes) {
                  try {
                  if (browser.getActiveProject() != null) {
                    if (browser.getActiveNode().getDisplayName().indexOf(".doml") != -1) {
                      return ActionDodsGenerator;
                    }                 }
                  return null;
                } catch (NullPointerException ex) {
                  return null;
                }
              }
            });

            //Dusan 25.01.03.
            //Setting right click on Node tab in ProjectView
            ProjectView.registerContextActionProvider(new ContextActionProvider() {
              public Action getContextAction(Browser browser, Node[] nodes) {
                try {
                if (browser.getActiveProject() != null) {
                  if (browser.getActiveNode().getDisplayName().indexOf(".doml") != -1) {
                    return ActionDodsGenerator;
                  }
                }
                return null;
              } catch (NullPointerException ex) {
                 return null;
              }
              }
            });
    }

    //Dusan 22.01.03.
    public static final BrowserAction ActionDodsGenerator = new BrowserAction (
      "DODS Generator", 'w', "DODS Generator",
      KelpIconSet.getSmallIcon(),
      "DODS Generator" ) {

          public void actionPerformed(Browser browser) {
            DodsTool dt = new DodsTool();
            dt.actionPerformed(browser);
          }

        public void update(Browser browser) {
            setEnabled(true);
        }
  };

    //
    public DodsTool() {
        putValue(BrowserAction.SMALL_ICON, KelpIconSet.getSmallIcon());
    }

    // implement: org.enhydra.kelp.jbuilder.toolAbstractBasicWizard
    public String getTitle() {
        return CoreDodsTool.getDefaultTitle();
    }

    // implement: org.enhydra.kelp.jbuilder.toolAbstractBasicWizard
    public String getDescription() {
        return res.getString("Dods_content");
    }

    // implement: com.borland.primetime.ide.BrowserAction
    public void actionPerformed(Browser browser) {

        PrimeProject      primeProject = null;
        OtterNode[]       inputNodes = new OtterNode[0];
        CoreDodsTool coreTool = null;

        setNativeProject((JBProject) browser.getProjectView().getActiveProject());
        if (getNativeProject() == null) {

            // do nothing
        } else {
            primeProject = new PrimeProject(getNativeProject());
            primeProject.save();
            inputNodes = primeProject.getAllInput();
            saveNodes(inputNodes);
            primeProject.setProjectJavaPath(getNativeProject());
            primeProject.setEnhydraPath(getNativeProject());
//            primeProject.setDODSGenerator(new DODSGenerator());

            if (verifyAllClassPaths(browser.getOwner())) {
        System.setProperty("DODS_HOME", primeProject.getEnhydraPath()+
          File.separator+"dods");
                coreTool = new CoreDodsTool();
                coreTool.addHelpListener(this);
                coreTool.setProject(primeProject);
                coreTool.showDialog(browser);
                browser.getProjectView().refreshTree();

                Project project = browser.getActiveProject();

    				 if ((project != null) && (project instanceof JBProject)) {
      				final JBProject jbProject = (JBProject)project;
      		      //String[] rootPath= primeProject.getSourcePathArray();
						//Url url = new Url(new File(rootPath[0]));
						String rootPath=primeProject.getDodsRootPath();
						Url url = new Url(new File(rootPath));
            		try {
              			doAddPackages(jbProject, url.getFile(), url);
              			doAddFiles(jbProject, url);
		            }
         	   catch (Exception ex) {
      	        ex.printStackTrace();
           	 }
         }

            }
        }
    }


    /**
   * Recursively add needed PackageNodes
   *
   * This routine uses recursive logic to locate all subdirectories under
   * the given root directory and (if not already present) add a PackageNode
   * for each of those subdirectories which is a legal package name and
   * has files under it.
   *
   * @param jbProject Project where nodes are to be added.
   * @param rootDir Root directory of the recursive search.
   * @param url Current directory to examine for subdirectories.
   */
  private void doAddPackages(JBProject jbProject,
                             String rootDir,
                             Url url) throws Exception {
    Url[] urls = VFS.getChildren(url, Filesystem.TYPE_DIRECTORY);
    if (urls.length > 0) {
      for (int j = 0; j < urls.length; j++) {
        String name = urls[j].getFileObject().getAbsolutePath();
        String[] files = (new File(name)).list();
        boolean isPackage = false;
        for (int i=0; i<files.length; i++) {
        	if(files[i].toLowerCase().endsWith(".java"))
        		isPackage = true;
        }
        name = name.substring(rootDir.length() + 1);
        if ((name.indexOf(' ') == -1)
          && (name.indexOf('-') == -1)
          && (name.indexOf('.') == -1)) {
          name = name.replace(File.separatorChar, '.');
       if(isPackage) {
          if (jbProject.findNodes(name).length == 0) {
            new PackageNode(jbProject, jbProject, name);
          }
        }
          doAddPackages(jbProject, rootDir, urls[j]);
        }
     }
    }
  }



  /**
   * Add needed FileNodes
   *
   * This routine finds all the files in the provided directory and adds
   * them to the given project if they are not there already.
   *
   * @param jbProject Project where nodes are to be added.
   * @param url Current directory to examine for subdirectories.
   */
  private void doAddFiles(JBProject jbProject,
                          Url url) throws Exception {
    Url[] urls = VFS.getChildren(url, Filesystem.TYPE_FILE);
    if (urls.length > 0) {
      for (int j = 0; j < urls.length; j++) {
     	 if(urls[j].getFile().toLowerCase().endsWith(".doml")) {
        if (jbProject.findNode(urls[j]) == null) {
          FileNode node = jbProject.getNode(urls[j]);
          node.setParent(jbProject);
        }
       }
      }
    }
  }


    public String getHelpAnchor() {
        return HELP_LINK;
    }
}
