/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.build;

// ToolBox imports
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;

// JBuilder imports
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.build.Builder;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.build.BuilderManager;
import com.borland.primetime.build.Phase;
import com.borland.jbuilder.node.JBProject;


// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.jbuilder.node.PrimeDocumentNode;
import org.enhydra.kelp.jbuilder.node.PrimeTextFileNode;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.jbuilder.node.PrimeTemplateNode;

/**
 * This class controls building and maintaining properties for HTML/XML files
 * using the Enhydra XMLC Compiler.
 *
 */
public class DodsBuilder extends Builder {

    public DodsBuilder() {}

    /**
     * Register open tool
     *
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        try {
          KelpInfo.verifyIDEClassPath();
          BuilderManager.registerBuilder(new DodsBuilder());
        } catch (Exception e) {
          System.err.println(e.getMessage());
        }
    }

    /**
     * Called by the Build System when make or rebuild occurs
     */
    public void updateBuildProcess(BuildProcess buildProcess,
                                   Node nativeNode) {


        OtterDocumentNode docNode = null;
        OtterProject      otterProject = null;
        TextFileNode      textNode = null;
        PathHandle        path = null;
        DodsBuildTask     task = null;

        task = (DodsBuildTask)buildProcess.getFirstBuildTask(DodsBuildTask.SHARED_KEY,DodsBuildTask.class);
        if (task == null) {

            task = new DodsBuildTask();

            // There is both a map and a list
            //buildProcess.putTask(DodsBuildTask.SHARED_KEY, task);

            // Add xmlc task to top of list so genrated DOM
            // classes are available when dependent classes
            // are compiled.
            //List list = buildProcess.getBuildTaskList();

            //list.add(0, task);
            buildProcess.createTask(DodsBuildTask.class,DodsBuildTask.SHARED_KEY);
        }
        task.increment();
        //
        if (nativeNode instanceof TextFileNode) {
            textNode = (TextFileNode) nativeNode;
            otterProject =
                new PrimeProject((JBProject) textNode.getProject());
      	  if(otterProject.isDodsBuild()) {
		buildProcess.addDependency(DodsBuildTask.SHARED_KEY, Phase.PRE_COMPILE_PHASE);
          }
        }
  }

}
