/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.node;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ToolException;

// JBuilder
import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.jbuilder.paths.PathSet;
import com.borland.jbuilder.paths.PathSetManager;
import com.borland.jbuilder.paths.ProjectPathSet;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.vfs.Url;
import com.borland.jbuilder.paths.PathSetManager;
import com.borland.primetime.runtime.RunConfiguration;
import com.borland.primetime.runtime.RunPropertyGroup;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.deployer.RunParameters;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.PropertyKeys;

// JDK
import java.io.File;
import java.util.ArrayList;
import java.lang.reflect.Method;

/**
 * Class declaration
 *
 *
 * @author
 */
public class PrimeProject extends OtterProject {
    private JBProject        nativeProject = null;
    private PrimeNodeFactory factory = null;

    /**
     * Constructor declaration
     *
     *
     * @param project
     */
    public PrimeProject(JBProject p) {
        nativeProject = p;
        factory = new PrimeNodeFactory();
    }

    public boolean isDefaultProject() {
        boolean def = false;

        if (getNativeProject() == null) {

            //
        } else {
            def = Browser.getDefaultProject().equals(getNativeProject());
        }
        return def;
    }

    public String getCodeGenDefaultDestination() {
        PathHandle ph = null;
        File       file = null;

        file = new File(getRootPath());
        if (file.getParentFile() == null) {
            ph = PathHandle.createPathHandle(file);
        } else {
            ph = PathHandle.createPathHandle(file.getParentFile());
        }
        return ph.getPath();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterTextFileNode getFirstPolicy() {
        PolicyNodeReader reader = new PolicyNodeReader();

        return reader.getFirstNode(nativeProject);
    }

    public String getWorkingPath() {
        String path = new String();
        File   file = null;

        path = getRootPath();
        path = nativeProject.getProperty(PropertyKeys.NAME_WORKING_DIR, path);
        if (path.indexOf("%|") == 1) {    // nores
            StringBuffer buf = new StringBuffer();

            buf.append(path.charAt(0));
            buf.append(':');
            buf.append(path.substring(3));
            path = buf.toString();
        }
        path = PathUtil.expandPathRelativeToProject(this, path);
        path = PathHandle.createPathString(path);
        return path;
    }

    public void setWorkingPath(String inPath) {
        PathHandle     handle = null;
        ProjectPathSet pathSet = null;
        Method         setWD = null;
        Method         save = null;
        Class[]        paramClasses = new Class[0];
        Url[]          paramValues = new Url[0];

        handle = PathHandle.createPathHandle(inPath);
        pathSet = nativeProject.getPaths();
        try {
            paramValues = new Url[1];
            paramValues[0] = new Url(handle.getFile());
            paramClasses = new Class[1];
            paramClasses[0] = paramValues[0].getClass();
            setWD = pathSet.getClass().getMethod("setWorkingDirectory",
                                                 paramClasses);    // nores
            setWD.invoke(pathSet, paramValues);
            paramClasses = new Class[0];
            paramValues = new Url[0];
            save = pathSet.getClass().getMethod("save", paramClasses);    // nores
            save.invoke(pathSet, paramValues);
        } catch (Exception e) {

            // if exception, assume jb3
            String compress = new String();

            compress = PathUtil.compressPathRelativeToProject(this,
                    handle.getPath());
            nativeProject.setProperty(PropertyKeys.NAME_WORKING_DIR,
                                      compress);
        }
    }

    /**
     * Get all nodes from input directory.
     * implements OtterProject
     */
    public OtterFileNode[] getAllInput() {
        String[]        ext = new String[0];
        OtterFileNode[] input = new OtterFileNode[0];
        OtterFileNode[] nodes = new OtterFileNode[0];
        ArrayList       list = new ArrayList();
        PathHandle      inputPath = null;
        PathHandle      cursorPath = null;

        inputPath = PathHandle.createPathHandle(getDeployInputPath());
        if (isDeployInputFilter()) {
            ext = new String[1];
            ext[0] = Constants.TYPE_IN;
        }
        nodes = findFileNodesByType(ext);
        for (int i = 0; i < nodes.length; i++) {
            cursorPath = PathHandle.createPathHandle(nodes[i].getFilePath());
            if (inputPath.parentOf(cursorPath)) {
                list.add(nodes[i]);
            }
        }
        input = new OtterFileNode[list.size()];
        input = (OtterFileNode[]) list.toArray(input);
        list.clear();
        return input;
    }

    /**
     * Method declaration
     *
     *
     * @param extensions
     *
     * @return
     */
    public OtterFileNode[] findFileNodesByType(String[] extensions) {
        NodeReader reader = new NodeReader(extensions);

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterTextFileNode[] getAllDeploymentDescs() {
        DeploymentNodeReader reader = new DeploymentNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterDocumentNode[] getAllDocuments() {
        DocumentNodeReader reader = new DocumentNodeReader();

        return reader.getNodes(this);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeProject() {
        return nativeProject;
    }

    /**
     * Method declaration
     *
     *
     * @param o
     */
    public void setNativeProject(Object o) {
        nativeProject = (JBProject) o;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeNode() {
        return getNativeProject();
    }

    /**
     * Method declaration
     *
     *
     * @param o
     */
    public void setNativeNode(Object o) {
        setNativeProject(o);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterProject getProject() {
        return this;
    }

    public OtterNode getParent() {
        return null;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterNodeFactory getNodeFactory() {
        return factory;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public boolean isOpenBuild() {
        return true;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getClassOutputPath() {
        String out = File.separator + Constants.DIR_CLASSES;

        if (nativeProject != null) {
            out = nativeProject.getPaths().getOutPath().getFile();
        }

        // Make match current OS.
        out.replace('\\', File.separatorChar);
        out.replace('/', File.separatorChar);
        return out;
    }

    public void setClassOutputPath(String p) {
        Url classOut = null;

        classOut = new Url(new File(p));
        nativeProject.getPaths().setOutPath(classOut);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getSourcePath() {
        StringBuffer path = new StringBuffer();
        Url          urlSet[];

        if (nativeProject != null) {
            urlSet = nativeProject.getPaths().getSourcePath();
            for (int i = 0; i < urlSet.length; i++) {
                if (path.length() > 0) {
                    path.append('/');
                }
                if (i > 0) {
                    path.append(';');
                }
                path.append(PathHandle.createPathHandle(urlSet[i].getFile()));
            }
        }
        return path.toString();
    }

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     */
    public void setSourcePathArray(String[] pathAsArray) {
        Url urlSet[] = new Url[pathAsArray.length];

        for (int i = 0; i < pathAsArray.length; i++) {
            urlSet[i] = new Url(new File(pathAsArray[i]));
        }
        nativeProject.getPaths().setSourcePath(urlSet);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String[] getSourcePathArray() {
        String[] pathAsArray = null;
        Url      urlSet[];

        if (nativeProject != null) {
            urlSet = nativeProject.getPaths().getSourcePath();
            pathAsArray = new String[urlSet.length];
            for (int i = 0; i < urlSet.length; i++) {
                pathAsArray[i] =
                    PathHandle.createPathString(urlSet[i].getFile());
            }
        }
        return pathAsArray;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getClassPath() {
        PathSet pathSet;
        Url     urlSet[];
        String  path = null;
        String  libs = null;

        if (nativeProject != null) {
            libs = getProperty(JBProject.PROPERTY_LIBRARIES);
            pathSet = PathSetManager.getLibrary(libs);
            urlSet = pathSet.getFullClassPath();
            path = Url.getPath(urlSet);
        }
        return path;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     */
    public String getProperty(String property) {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getProperty(property);
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, String value) {
        if (nativeProject != null) {
            if (property.trim().equalsIgnoreCase(PropertyKeys.NAME_LIBRARIES)
                    && value.trim().equalsIgnoreCase(PropertyKeys.VALUE_ENHYDRA)) {
                nativeProject.getPaths().putClassOnFullPath(ToolBoxInfo.getEnhydraMainClass());
            } else {
                nativeProject.setProperty(property, value);
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param i
     */
    public void setProperty(String property, int i) {
        String value = Integer.toString(i);

        setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getRootPath() {
        PathHandle ph = null;

        ph =
            PathHandle.createPathHandle(nativeProject.getUrl().getFileObject());
        ph = ph.getParent();
        return ph.getPath();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public char getFileSeparatorChar() {
        return '/';
    }

    /**
     * Method declaration
     *
     */

    public void configureRunClass() {

        RunParameters param = new RunParameters(this);

        // strings not to be resourced
		  nativeProject.setProperty("runtime.0","RunnableType",    // nores
        "com.borland.jbuilder.runtime.ApplicationRunner");

        nativeProject.setProperty("runtime.0", "application.class",    // nores
        ToolBoxInfo.getEnhydraMainClass());
        nativeProject.setProperty("runtime.0",
                                  "application.parameters",       // nores
        							param.getAppParameters());
        //DACHA 17.05.2003
       	nativeProject.setProperty("runtime.0",    // nores
            "application.vmparameters",               // nores
            param.getVMParameters());
        //END DACHA
        
        if (!ToolBoxInfo.isEnhydra3()) {
            nativeProject.setProperty("runtime.0",    // nores
            "application.vmparameters",               // nores
            param.getVMParameters());
        }
        RunPropertyGroup.setConfigurationCount(nativeProject,0);
        RunConfiguration rc = new RunConfiguration(nativeProject,0);
//        rc.setName("my configuration");
        rc.readProperties();
        rc.writeProperties();

    }

    public void save() {
        Project project = null;

        if (getNativeNode() instanceof Project) {
            project = (Project) getNativeNode();
            try {
                project.save();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }


    /**
     * Method declaration
     *
     *
     * @return
     */
    public void setProjectJavaPath(JBProject jbproj) {

      String javaPath =
          ""+PathSetManager.getJDK(jbproj.getProperty("JDK")).
            getHomePath();
      if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
        javaPath = javaPath.substring(javaPath.indexOf("%|")-1,
          javaPath.length());
        javaPath = javaPath.replace('%',':');
        javaPath = javaPath.substring(0, javaPath.indexOf("|"))+
          javaPath.substring(javaPath.indexOf("|")+1, javaPath.length());;
      }

        setProperty(DODS_PROJECT_JAVA_PATH, javaPath);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public void setEnhydraPath(JBProject jbproj) {
        String javaPath =
          PathSetManager.getLibrary("Enhydra").getClassPath()[0].getFile();
        javaPath = javaPath.substring(1, javaPath.indexOf("/lib"));
        setProperty(DODS_ENHYDRA_PATH, javaPath);
    }

    }


/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
class DeploymentNodeReader {
    private ArrayList list = new ArrayList();

    /**
     * Constructor declaration
     *
     */
    protected DeploymentNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     */

    protected OtterTextFileNode[] getNodes(JBProject nativeProject) {
        OtterTextFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterTextFileNode[list.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterTextFileNode) list.get(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     */
    private void recursiveRead(Node container) {
        String     filename = null;
        Node[]     nativeNodes = container.getChildren();
        PathHandle current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            TextFileNode nativeFile = null;

            if (nativeNodes[i] instanceof TextFileNode) {
                nativeFile = (TextFileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = PathHandle.createPathHandle(filename);
                if (current.isFile()
                        && (current.endsWith(File.separator + Constants.FILE_WEB_XML)
                            || current.endsWith(File.separator
                                                + Constants.FILE_EJB_JAR_XML))) {
                    PrimeTextFileNode primeNode = null;

                    primeNode = new PrimeTextFileNode(nativeFile);
                    list.add(primeNode);
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
class DocumentNodeReader {
    private ArrayList    list = new ArrayList();
    private OtterProject otterProject = null;
    private String[]     docTypes = new String[0];

    /**
     * Constructor declaration
     *
     */
    protected DocumentNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     */
    protected OtterDocumentNode[] getNodes(OtterProject p) {
        OtterDocumentNode[] nodes = new OtterDocumentNode[0];

        otterProject = p;
        docTypes = otterProject.getDocTypes();
        list.clear();
        recursiveRead((JBProject) otterProject.getNativeProject());
        nodes = new OtterDocumentNode[list.size()];
        nodes = (OtterDocumentNode[]) list.toArray(nodes);
        list.clear();
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     */
    private void recursiveRead(Node container) {
        Node[]     nativeNodes = container.getChildren();
        PathHandle current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            FileNode nativeFile = null;

            if (nativeNodes[i] instanceof FileNode) {
                nativeFile = (FileNode) nativeNodes[i];
                current =
                    PathHandle.createPathHandle(nativeFile.getUrl().getFileObject());
                if (current.getFile().exists()
                        && current.hasExtension(docTypes)) {
                    OtterDocumentNode docNode = null;

                    docNode = new PrimeDocumentNode(nativeFile);
                    list.add(docNode);
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}
class NodeReader {
    private ArrayList list = new ArrayList();
    private String[]  extensions = new String[0];

    /**
     * Constructor declaration
     *
     *
     * @param exts
     */
    protected NodeReader(String[] exts) {
        extensions = exts;
    }

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     */
    protected OtterFileNode[] getNodes(JBProject nativeProject) {
        OtterFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterFileNode[list.size()];
        for (int i = 0; i < nodes.length; i++) {
            if (list.get(i) instanceof OtterTextFileNode) {
                nodes[i] = (OtterTextFileNode) list.get(i);
            } else {
                nodes[i] = (OtterFileNode) list.get(i);
            }
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     */
    private void recursiveRead(Node container) {
        String     filename = null;
        Node[]     nativeNodes = container.getChildren();
        PathHandle current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            FileNode nativeFile = null;

            if (nativeNodes[i] instanceof FileNode) {
                nativeFile = (FileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = PathHandle.createPathHandle(filename);
                if (current.getFile().exists()) {
                    if (current.hasExtension(extensions)
                            || (extensions.length == 0)) {
                        PrimeNode primeNode = null;

                        if (nativeFile instanceof TextFileNode) {
                            primeNode =
                                new PrimeTextFileNode((TextFileNode) nativeFile);
                        } else {
                            primeNode = new PrimeNode(nativeFile);
                        }
                        list.add(primeNode);
                    }
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
class PolicyNodeReader {
    private OtterTextFileNode node = null;

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     */
    protected OtterTextFileNode getFirstNode(JBProject nativeProject) {
        recursiveRead(nativeProject);
        return node;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     */
    private void recursiveRead(Node container) {
        String     filename = null;
        Node[]     nativeNodes = container.getChildren();
        PathHandle current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            TextFileNode nativeFile = null;

            if (nativeNodes[i] instanceof TextFileNode) {
                nativeFile = (TextFileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = PathHandle.createPathHandle(filename);
                if (current.isFile()
                        && current.endsWith(Constants.FILE_JAVA_POLICY)) {
                    node = new PrimeTextFileNode(nativeFile);
                    break;
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}
