/*
* Enhydra Java Application Server Project
*
* The contents of this file are subject to the Enhydra Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License on
* the Enhydra web site ( http://www.enhydra.org/ ).
*
* Software distributed under the License is distributed on an "AS IS"
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
* the License for the specific terms governing rights and limitations
* under the License.
*
* The Initial Developer of the Enhydra Application Server is Lutris
* Technologies, Inc. The Enhydra Application Server and portions created
* by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
* All Rights Reserved.
 */

/*
*
* @author    Tanja Jovanovic, Nenad Vico
* @version   1.0.0
*
 */
package org.enhydra.kelp.ant.dods;

import java.io.File;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import org.enhydra.dods.wizard.DefaultDODSWizard;
import org.enhydra.dods.generator.DODSGenerator;
import org.enhydra.dods.Common;
import org.enhydra.dods.generator.DODSGenerateException;
import org.enhydra.kelp.ant.node.AntProject;

/**
 * This class is used for generating DODS code and DODS documentation.
 */
public class KelpDODSGenerator extends DODSGenerator {

    public static AntProject project = null;

    public KelpDODSGenerator(){
        super();
        setForce("true");
        setKelp(true);
    }

    /**
     * This method is called before wizard closing. User should
     * override this method to involve any pre-closing action.
     */
    public void preClose() {
        saveParameters();
        System.out.println("Kelp.preClose()");
    }

    public static void main(String[] args) {

        KelpDODSGenerator kelpGenerator = new KelpDODSGenerator();

        if (args.length > 0) {
            // check path
            File root = new File(args[0]);
            try {
                if(!root.exists()){
                    System.err.println("ERROR: directory'"+args[0]+"' don't exist!");
                }

                project = new AntProject(args[0]);
//                String enhydraDir = project.getProperty(AntProject.ENHYDRA_DIR);
//                File dodsDirFile = new File(enhydraDir,"dods");
//                System.setProperty("DODS_HOME", dodsDirFile.getAbsolutePath());

//                System.setProperty("JAVA_HOME", System.getProperty("java.home"));

            }
            catch (Exception ex) {
                project = null;
            }
            if(project != null){

                String domlFilePath = project.getProperty(AntProject.DODS_DOML);
                if(domlFilePath.endsWith(".doml") || domlFilePath.endsWith(".DOML")){
                    kelpGenerator.setDoml(domlFilePath);
                }
                if(project.getProperty(AntProject.DODS_INVOKE).equalsIgnoreCase("true")){
                    kelpGenerator.setInvoke(true);
                }else{
                    kelpGenerator.setInvoke(false);
                }
                if ((project.getProperty(AntProject.DODS_GENERATED_SRC_DIR) != null)) {
                    kelpGenerator.setOutputDir(project.getProperty(AntProject.DODS_GENERATED_SRC_DIR));
                }
                kelpGenerator.setTemplateSet(project.getProperty(AntProject.DODS_TEMPLATE_SET));
                kelpGenerator.setAction(project.getProperty(AntProject.DODS_ACTION));
                kelpGenerator.runWizard();
            }else{
                System.err.println("ERROR: Could not find project root!");
            }
        }

    }

    private void saveParameters(){
        project.setProperty(AntProject.DODS_ACTION, this.getAction());
        project.setProperty(AntProject.DODS_DOML,this.getDoml());
        project.setProperty(AntProject.DODS_GENERATED_SRC_DIR,this.getOutputDir());
        if(this.getInvoke()){
            project.setProperty(AntProject.DODS_INVOKE,"true");
        }else{
            project.setProperty(AntProject.DODS_INVOKE,"false");
        }
        project.setProperty(AntProject.DODS_TEMPLATE_SET,this.getTemplateSet());
        project.save();
    }
}