package org.enhydra.kelp.ant.xmlc;
//Kelp
import org.enhydra.kelp.common.*;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.ant.AntFacade;

//Toolbox
import org.enhydra.tool.common.ToolException;

//java
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.ant.node.AntProject;
import java.io.FileWriter;
import java.io.BufferedWriter;

/**
 * <p>Title: </p>
 * <p>Description: This class execute XML compiler using <code>Ant tool</code>
 * and target <code>xmlc</code> defined in file <code>build.xml</code> </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Damir Milovic
 * @version 1.0
 */

public class AntXMLCBuilder extends AbstractEchoBuilder {

  public AntXMLCBuilder() {
      super();
  }

  public AntXMLCBuilder(WriteListener listener) {
      super(listener);
  }

  public AntXMLCBuilder(WriteListener[] listeners) {
    super();
    for (int i = 0; i < listeners.length; i++) {
      addWriteListener(listeners[i]);
    }
  }

  public AntXMLCBuilder(WriteListener listener, boolean buffer) {
      super(listener, buffer);
  }

  //Implementation
  protected void buildImpl() throws org.enhydra.tool.common.ToolException {
    File root = new File(getProject().getWorkingPath());
    try{
      String enhydraDir = getProject().getProperty(AntProject.ENHYDRA_DIR);
      String execLine = enhydraDir + "/bin/ant";
      if (System.getProperty("os.name").toLowerCase().startsWith("windows")) {
          execLine += ".bat";
      }
      Process process = Runtime.getRuntime().exec(execLine + " xmlc", null, root);
      InputStream inputStream = process.getInputStream();
      InputStream errorStream = process.getErrorStream();
      BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
      BufferedReader errorBufferedReader = new BufferedReader(new InputStreamReader(errorStream));
      // init log (ouput) file:
      File logFile = null;
      FileWriter logFileWriter = null;
      BufferedWriter logBuffer = null;
      PrintWriter logPrint = null;
      boolean logFileOK = false;
      String logFileName = ((AntProject)getProject()).getOutputFilename();
      if (logFileName != null) {
        logFile = new File(logFileName);
      }
      if ((logFile != null) && (!logFile.isDirectory())
            && ((AntProject)getProject()).isOutputFileEnabled()) {
          if (logFile.length() > 1e6) {
              logFileWriter = new FileWriter(logFile, false /* overwrite */ );
          } else {
              logFileWriter = new FileWriter(logFile, true /* append */ );
          }
          logBuffer = new BufferedWriter(logFileWriter, 4096);
          logPrint = new PrintWriter(logBuffer, false /* auto flush */);
          logFileOK = true;
      }

      String s = null;
      String er = null;
      while(((s = bufferedReader.readLine()) != null) || ((er = errorBufferedReader.readLine()) != null)){
        if(s != null){
          getEchoWriter().println(s);
          if (logFileOK) {
              logPrint.println(s);
          }
          s = null;
        }
        if(er != null){
          getEchoWriter().println(er);
          if (logFileOK) {
              logPrint.println(er);
          }
          er = null; //must reset becouse second condition is not executed if firs condition is true
        }
      }
      if (logFileOK) {
          logPrint.flush();
          logPrint.close();
      }

    }catch(IOException e){
      throw new ToolException(e,"XML Compile error: "+e.toString());
    }
  }
}