/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */

package org.enhydra.kelp.common.bridge;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;

// XMLC imports
import org.w3c.dom.Document;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.compiler.EditDOM;
import org.enhydra.xml.xmlc.metadata.MetaData;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

//
public class XMLCConnectionFactory {

    // string not to be resourced
    private static final String EDIT_DOM_CLASS =
        "org.enhydra.xml.xmlc.compiler.EditDOM"; // nores
    private static final String ADD_URL_EDITS_METHOD = "addURLEdits"; // nores
    private static final String ADD_DELETE_CLASSES_METHOD =
        "addDeleteClasses"; // nores

    //
    public XMLCConnectionFactory() {}

    public static MetaDataHandler createMetaDataHandler() {
        MetaDataHandler handler = null;

        if (ToolBoxInfo.isXMLCVersion(2)) {
            handler = new MetaDataHandlerV2();
        } else {
            handler = new MetaDataHandlerV1();
        }
        return handler;
    }

    public static EditDOM createEditDOM(MetaDataHandler h) {
        EditDOM editDOM = null;

        if (ToolBoxInfo.isXMLCVersion(2)) {
            editDOM = new EditDOM((MetaData) h.getMetaData());
        } else {
            Class       editClass = null;
            Class[]     params = new Class[0];
            Constructor construct = null;
            Method[]    meths = new Method[0];
            Object[]    values = new Object[0];

            try {
                editClass =
                    Class.forName(XMLCConnectionFactory.EDIT_DOM_CLASS);
                construct = editClass.getConstructor(params);
                editDOM = (EditDOM) construct.newInstance(params);
                meths = editClass.getMethods();
                for (int i = 0; i < meths.length; i++) {
                    if (meths[i].getName().equalsIgnoreCase(XMLCConnectionFactory.ADD_URL_EDITS_METHOD)) {
                        values = new Object[1];
                        values[0] = h.getURLMappings();
                        meths[i].invoke(editDOM, values);
                        break;
                    }
                }
                for (int i = 0; i < meths.length; i++) {
                    if (meths[i].getName().equalsIgnoreCase(XMLCConnectionFactory.ADD_DELETE_CLASSES_METHOD)) {
                        values = new Object[1];
                        values[0] = h.getDeleteElements();
                        meths[i].invoke(editDOM, values);
                        break;
                    }
                }
            } catch (Exception e) {
                editClass = null;
                e.printStackTrace();
            }
        }
        return editDOM;
    }

    public static PrintInfo createPrintInfo(Document doc,
                                            XMLCDocument xmlcDoc) {
        PrintInfo pi = null;

        if (ToolBoxInfo.isXMLCVersion(2)) {
            pi = new PrintInfoV2(doc, xmlcDoc);
        } else {
            pi = new PrintInfoV1(doc, xmlcDoc);
        }
        return pi;
    }

    public static Generator createGenerator(MetaDataHandler h,
                                            XMLCDocument doc,
                                            PrintWriter writer) throws XMLCException {
        Generator gen = null;

        if (ToolBoxInfo.isXMLCVersion(2)) {
            gen = new GeneratorV2(h, doc, writer);
        } else {
            gen = new GeneratorV1(h, doc, writer);
        }
        return gen;
    }

    public static Parser createParser(PrintWriter traceWriter,
                                      MetaDataHandler metaData) {
        Parser parser = null;

        if (ToolBoxInfo.isXMLCVersion(2)) {
            parser = new ParserV2(traceWriter, metaData);
        } else {
            parser = new ParserV1(traceWriter, metaData);
        }
        return parser;
    }

}
