/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.ExtensionPanel;
import org.enhydra.kelp.common.DirectoryFilter;

// ToolBox
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
//Dusan 24.11.2002.
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.io.File;
import java.io.PrintWriter;
import java.lang.ref.WeakReference;
import java.util.Arrays;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

//Dusan 24.11.2002.
// Standard imports
import java.awt.event.*;
import java.io.File;
import java.util.ResourceBundle;
// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFileNode;

//
//
public class ContentPanel extends JPanel implements Instructor, DeployStep {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private GridBagLayout       layoutMain;
    private JTabbedPane         tab;
    private JPanel              panelFiller;
    private JPanel              panelPaths;
    private ExtensionPanel      panelTypes;
    private GridBagLayout       layoutPaths;
    private GridBagLayout       layoutTypes;
    private JPanel              panelSource;
    private JTextField          textSource;
    private GridBagLayout       layoutSource;
    private JButton             buttonSource;
    private JPanel              panelDoc;
    private GridBagLayout       layoutDoc;
    private JTextField          textDoc;
    private JPanel              panelPathsFiller;
    private JCheckBox           checkEnable;
    private LocalButtonListener buttonListener = null;
    private LocalCheckListener  checkListener = null;
    private WeakReference       projectRef = null;
    private boolean             ran = false;

    public ContentPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // implements Instructor
    public String getTab() {
        return "Content";
    }

    // implements Instructor
    public String getTitle() {
        return "Web content";
    }

    // implements Instructor
    public String getInstructions() {
        return "The deployer will copy content types from the content source"
               + " directory to the archive document root. Use the archive document "
               + " root to select content when building web application archives.";
    }

    // implements DeployStep
    public void refresh() {
        if (getProject() != null) {
            checkEnable.setSelected(getProject().isDeployContent());
        }
        privateRefresh();
    }

    private void privateRefresh() {
        boolean enable = checkEnable.isSelected();

        buttonSource.setEnabled(enable);
        panelTypes.setEnabled(false);
        for (int i = 0; i < tab.getTabCount(); i++) {
            tab.setEnabledAt(i, enable);
        }
        if (getProject() != null) {
            textDoc.setText(PathUtil.getDeployContentPath(getProject(),
                    getProject().getDeployRootPath()));
            textDoc.setToolTipText(textDoc.getText());
        }
    }

    // implements DeployStep
    public void clearAll() {
        checkEnable.removeChangeListener(checkListener);
        buttonSource.removeActionListener(buttonListener);
        panelTypes.clearAll();
        if (projectRef != null) {
            projectRef.clear();
        }
        removeAll();
        projectRef = null;
        checkListener = null;
        buttonListener = null;
    }

    // implements DeployStep
    public boolean isDataValid() {
        return DeployStepUtil.isDataValid(this);
    }

    // implements DeployStep
    public boolean isDataEqual(OtterProject project) {
        PathHandle projectPath = null;
        PathHandle panelPath = null;
        String[]   panelList = null;
        String[]   projectList = null;
        boolean    equal = true;

        projectRef = new WeakReference(project);
        if (project == null) {
            System.err.println("ContentPanel.isDataEqual() - project null");
            return true;
        } else if (checkEnable.isSelected()
                   == project.isDeployContent()) {
            equal = false;
        }
        if (equal) {
            projectPath =
                PathHandle.createPathHandle(project.getDeployResourcePath());
            panelPath = PathHandle.createPathHandle(textSource.getText());
            if (!projectPath.equals(panelPath)) {
                equal = false;
            }
        }
        if (equal) {
            panelList = panelTypes.getExtensions();
            Arrays.sort(panelList);
            projectList = project.getContentTypes();
            Arrays.sort(projectList);
            equal = Arrays.equals(panelList, projectList);
        }
        return equal;
    }

    // implements DeployStep
    public void validateData() throws DataValidationException {
        PathHandle path = null;

        path = PathHandle.createPathHandle(textSource.getText());
        if (checkEnable.isSelected()) {
            if (!path.getPath().equalsIgnoreCase(textSource.getText())) {
                throw new DataValidationException("Invalid content source directory: "
                                                  + textSource.getText());
            }
            if (!path.isDirectory()) {
                throw new DataValidationException("Invalid content source directory: "
                                                  + path.getPath());
            }
            if (panelTypes.getExtensions().length == 0) {
                throw new DataValidationException("No content types defined");
            }
        }
    }

    // implements DeployStep
    public boolean isBuilt(OtterProject project) {
        projectRef = new WeakReference(project);
        ran = DeployStepUtil.isBuilt(this, ran, project);
        return ran;
    }

    // implements DeployStep
    public boolean isSelectable(OtterProject project, DeployStep[] steps) {
        boolean able = false;

        projectRef = new WeakReference(project);
        for (int i = 0; i < steps.length; i++) {
            if (steps[i] instanceof GeneralPanel) {
                able = steps[i].isBuilt(project);
                break;
            }
        }
        return able;
    }

    // implements DeployStep
    public void build(OtterProject project,
                      WriteListener listener) throws DataValidationException {
        ContentBuilder builder = null;

        projectRef = new WeakReference(project);
        if (!isBuilt(project)) {
            write(project);
            builder = new ContentBuilder(listener);
            builder.setEcho(true);
            builder.setProject(project);
            builder.buildInCurrentThread();
            ran = true;
        }
    }

    // implements DeployStep
    public void read(OtterProject project) {
        PathHandle path = null;

        projectRef = new WeakReference(project);
        checkEnable.setSelected(project.isDeployContent());
        path = PathHandle.createPathHandle(project.getDeployResourcePath());
        if (path.isDirectory()) {
            textSource.setText(path.getPath());
        } else {
            textSource.setText(new String());
        }
        textSource.setToolTipText(textSource.getText());
        textDoc.setText(PathUtil.getDeployContentPath(project,
                textSource.getText()));
        textDoc.setToolTipText(textDoc.getText());
        panelTypes.setExtensions(project.getContentTypes());
        panelTypes.setDefaults(PropUtil.getDefaultContentTypes());
        privateRefresh();
    }

    // implements DeployStep
    public void write(OtterProject project) throws DataValidationException {
        projectRef = new WeakReference(project);
        validateData();
        if (!isDataEqual(project)) {
            project.setDeployContent(checkEnable.isSelected());
            project.setDeployResourcePath(PathHandle.createPathString(textSource.getText()));
        }
        if (project == null) {
            System.err.println("ContentPanel.write() - project null");
        } else {
            project.setContentTypes(panelTypes.getExtensions());
        }
    }

    //
    private void chooseSourceDirectory() {
        File       choice = null;
        PathHandle path = null;

        choice =
            SwingUtil.getDirectoryChoice(this, textSource.getText(),
                                         "Select content source directory");
        path = PathHandle.createPathHandle(choice);
        if (path.isDirectory()) {
            textSource.setText(path.getPath());
            textSource.setToolTipText(textSource.getText());
            privateRefresh();
        }
    }

    private OtterProject getProject() {
        OtterProject project = null;

        if (projectRef != null) {
            project = (OtterProject) projectRef.get();
        }
        return project;
    }

    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonSource.addActionListener(buttonListener);
        checkListener = new LocalCheckListener();
        checkEnable.addChangeListener(checkListener);
    }

    private void jbInit() throws Exception {
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelPaths = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        panelTypes =
            (ExtensionPanel) Beans.instantiate(getClass().getClassLoader(),
                                               ExtensionPanel.class.getName());
        layoutPaths =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutTypes =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelSource = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        textSource =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        layoutSource =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonSource =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelDoc = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        layoutDoc =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textDoc = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                 JTextField.class.getName());
        panelPathsFiller =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        checkEnable =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        panelPaths.setLayout(layoutPaths);
        panelTypes.setLayout(layoutTypes);
        panelSource.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(new Color(187, 218, 252), new Color(91, 107, 123)),
                                               "Content source"));
        panelSource.setLayout(layoutSource);
        textSource.setEnabled(false);
        textSource.setEditable(false);
        textSource.setText("Source ROOT");
        buttonSource.setText("...");
        panelDoc.setLayout(layoutDoc);
        panelDoc.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(new Color(187, 218, 252), new Color(91, 107, 123)),
                                            "Archive document root"));
        textDoc.setEnabled(false);
        textDoc.setEditable(false);
        textDoc.setText("DOCUMENT ROOT");
        checkEnable.setSelected(true);
        checkEnable.setText("Enable content deployment");
        panelPaths.add(panelSource,
                       new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(5, 5, 5, 5), 0, 0));
        panelSource.add(textSource,
                        new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 5, 0), 0, 0));
        panelSource.add(buttonSource,
                        new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelPaths.add(panelDoc,
                       new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(5, 5, 5, 5), 0, 0));
        panelDoc.add(textDoc,
                     new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 0), 0, 0));
        panelPaths.add(panelPathsFiller,
                       new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.BOTH,
                                              new Insets(0, 0, 0, 0), 0, 0));
        tab.add(panelPaths, "Paths");
        tab.add(panelTypes, "Types");
        this.setLayout(layoutMain);
        this.add(tab,
                 new GridBagConstraints(0, 1, 3, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(panelFiller,
                 new GridBagConstraints(0, 2, 1, 1, 0.0, 0.2,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.VERTICAL,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(checkEnable,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 2, 10), 0, 0));
    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonSource) {
//Dusan 24.11.2002.
//                chooseSourceDirectory();
                browseForDirectory();
            }
        }

    }
    private class LocalCheckListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            Object source = e.getSource();

            if (source == checkEnable) {
                if (getProject() != null) {
                  getProject().setDeployContent(checkEnable.isSelected());
                }
                privateRefresh();
            }
        }

    }

    /**
     * Method declaration
     *
     */
    private void browseForDirectory() {
        JFileChooser    chooser;
        File            file;
        String          fileDir = new String();
        String          fileName = new String();

        chooser = new JFileChooser();
        if (textSource.getText().trim().length() > 0) {
            chooser.setCurrentDirectory(new File(textSource.getText()));
        } else {
            PathHandle ph = null;

            chooser.setCurrentDirectory(new File(""));
        }
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setFileFilter((javax.swing.filechooser.FileFilter) new DirectoryFilter());
        chooser.setDialogTitle(res.getString("chooser_ContentRoot_DialogTitle"));
        chooser.setApproveButtonText(res.getString("OK"));
        int v = chooser.showOpenDialog(this);

        this.requestFocus();
        buttonSource.requestFocus();
        if (v == JFileChooser.APPROVE_OPTION) {
            if (chooser.getSelectedFile() == null) {
                textSource.setText(new String());
            } else {
                fileDir = chooser.getCurrentDirectory().toString();
                fileDir = stripSlash(fileDir);
                fileName = chooser.getSelectedFile().getName();
                String strDeployRoot = fileDir + File.separator + fileName;
                textSource.setText(strDeployRoot.replace('\\','/'));
            }
        }
        chooser.removeAll();
        chooser = null;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @return
     */
    private String stripSlash(String dir) {
        String stripped = new String(dir);

        if (dir != null) {
            if (dir.length() > 0) {
                if (dir.endsWith(File.separator)) {
                    stripped = dir.substring(0, dir.length() - 1);
                }
            }
        }
        return stripped;
    }
}
