/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.dods;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.swing.OutputPanel;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.DirectoryFilter;

// ToolBox
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;


// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.PrintWriter;
import java.lang.ref.WeakReference;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.beans.*;

// Standard imports
import java.awt.event.*;
import java.io.File;
import java.util.ResourceBundle;

//
public class GeneralPanel extends JPanel implements Instructor, DodsStep {

    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private GridBagLayout       layoutMain;
    private JPanel              panelDomlFile;
    private GridBagLayout       layoutDomlFile;
    private JTextField          textDomlFile;
    private JButton             buttonDomlFile;
    private JPanel              panelRootPath;
    private GridBagLayout       layoutRootPath;
    private JTextField          textRootPath;
    private JButton             buttonRootPath;
    private JPanel              panelMessage;
    private JCheckBox           checkSqlGenerate;
    private JButton             buttonMessage;
    private JCheckBox           checkSqlSplitter;
    private JCheckBox           checkJavaGenerate;
    private JCheckBox           checkBuild;
    private JTabbedPane         tabMessage;
    private JPanel              panelMessageOption;
    private OutputPanel         outputPanel;
    private GridBagLayout       layoutMessageOption;
    private BorderLayout        layoutMessage;
    private CheckBoxListener    checkBoxListener = null;
    private boolean             ran = false;
    private JPanel              panelType;
    private JComboBox           comboType;
    private WeakReference       projectRef = null;
    private GridBagLayout       layoutType;
    private LocalButtonListener buttonListener = null;

    private JLabel jTemplateTypeLabel = new JLabel();
    private JCheckBox jCheckBoxMultiDB = new JCheckBox();
    private JCheckBox jCheckBoxWebDocWF = new JCheckBox();
    private JPanel panelExtensions;
    private GridBagLayout       layoutExtensions;

    public GeneralPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // implements Instructor
    public String getTab() {
        return "General";
    }

    // implements Instructor
    public String getTitle() {
        return "Dods generator";
    }

    // implements Instructor
    public String getInstructions() {
        return "Please select Doml file and choose action.";
    }

    // implements DodsStep
    public void refresh() {

        // nothing to refresh
    }

    // implements DodsStep
    public void clearAll() {
        outputPanel.clearAll();
        removeAll();
    }

    // implements DodsStep
    public void read(OtterProject project) {
        PathHandle path = null;
        int        type = -1;
        String projectRoot = project.getSourcePath();
        if(projectRoot.indexOf(";")!=-1)
          projectRoot = projectRoot.substring(0, projectRoot.indexOf(";"));
        path = PathHandle.createPathHandle(projectRoot);

        if (isValidDeployRoot(path)) {
            textRootPath.setText(path.getPath());
        } else {
            textRootPath.setText(new String());
        }

        textRootPath.setToolTipText(textRootPath.getText());

        PathHandle domlPath = null;
        String[] ext = {"doml","DOML"};
        OtterFileNode[] domlFileNodes = project.findFileNodesByType(ext);
        String domlFileName= null;
        if(domlFileNodes.length > 0)
        	{
        		domlFileName = domlFileNodes[0].getFilePath();
        	}
        if(domlFileName!=null)
			{
        		domlPath = PathHandle.createPathHandle(domlFileName);
        		if (domlPath.isFile()) {
            	textDomlFile.setText(domlPath.getPath());
        		} else {
            	textDomlFile.setText(new String());
       	 }
        textDomlFile.setToolTipText(textDomlFile.getText());
       }

       //Dusan 24.01.2003.
        if (project.isOpenBuild()) {

            // Primetime
            checkBuild.setEnabled(true);
            checkBuild.setSelected(project.isDodsBuild());
        } else {

            // Win32 and JDeveloper
            checkBuild.setEnabled(false);
            checkBuild.setSelected(false);
        }

      }

    // implements DodsStep
    public void write(OtterProject project) throws DataValidationException {

        projectRef = new WeakReference(project);

     project.setDomlFilePath(textDomlFile.getText());


     project.setDodsRootPath(textRootPath.getText());

    //Dusan 12.02.2003.
      project.setExstensionParameter("");
    if (jCheckBoxMultiDB.isSelected() && jCheckBoxWebDocWF.isSelected()) {
      project.setExstensionParameter("mdbwdwf");
    } else {
      if (jCheckBoxMultiDB.isSelected())
        project.setExstensionParameter("mdb");
      if (jCheckBoxWebDocWF.isSelected())
        project.setExstensionParameter("wdwf");
    }

      if (checkSqlGenerate.isSelected() && !checkSqlSplitter.isSelected() &&
        !checkJavaGenerate.isSelected()) {
	    project.setDodsParameters("dods:sql");
      }

      if (checkSqlGenerate.isSelected() && checkSqlSplitter.isSelected() &&
        !checkJavaGenerate.isSelected()) {
	    project.setDodsParameters("dods:sqlsplit");
      }

      if (!checkSqlGenerate.isSelected() && checkJavaGenerate.isSelected()) {
          project.setDodsParameters("dods:javaNoCompile");
      }

      if (checkSqlGenerate.isSelected() && checkJavaGenerate.isSelected()) {
        if (!checkSqlSplitter.isSelected())
          project.setDodsParameters("dods:noCompile");
        else
          project.setDodsParameters("dods:noCompileSplit");
      }

      //Dusan 24.01.2003.
        if (project.isOpenBuild()) {
            project.setDodsBuild(checkBuild.isSelected());
        } else {
            // Win32 and JDeveloper
            project.setDodsBuild(false);
        }
  }

    //
    protected WriteListener getWriteListener() {
        return outputPanel;
    }

    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[7];

        comps[0] = buttonDomlFile;
        comps[1] = jCheckBoxMultiDB;
        comps[2] = jCheckBoxWebDocWF;
        comps[3] = checkSqlGenerate;
        comps[4] = checkSqlSplitter;
        comps[5] = checkJavaGenerate;
        comps[6] = checkBuild;
        return comps;
    }

    //
    private boolean isValidDodsRoot(PathHandle ph) {
        boolean valid = false;

        if (!ph.isFile()) {
            if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    private boolean isValidLogFile(PathHandle ph) {
        boolean valid = false;

        if (ph.hasExtension("txt")) {
            if (ph.isFile()) {
                valid = true;
            } else if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    //
    private void chooseDodsRoot() {
        File       choice = null;
        PathHandle path = null;

        choice =
            SwingUtil.getDirectoryChoice(this, textDomlFile.getText(),
                                         "Select Dods root directory");
        path = PathHandle.createPathHandle(choice);
        if (path.isDirectory()) {
            textDomlFile.setText(path.getPath());
            textDomlFile.setToolTipText(textDomlFile.getText());
        }
    }

    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonRootPath.addActionListener(buttonListener);
        buttonDomlFile.addActionListener(buttonListener);
        buttonMessage.addActionListener(buttonListener);

        checkBoxListener = new CheckBoxListener();
        checkSqlGenerate.addActionListener(checkBoxListener);
        checkJavaGenerate.addActionListener(checkBoxListener);
        checkBuild.addActionListener(checkBoxListener);
        //Dusan 12.02.2003.
        jCheckBoxMultiDB.addActionListener(checkBoxListener);
        jCheckBoxWebDocWF.addActionListener(checkBoxListener);
   }

    private void jbInit() throws Exception {
        panelRootPath =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutRootPath =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textRootPath =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonRootPath =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelDomlFile =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutDomlFile =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textDomlFile =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonDomlFile =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelMessage = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        //Dusan 12.02.2003.
        panelExtensions =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutExtensions =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        jCheckBoxMultiDB =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        jCheckBoxWebDocWF =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());

        checkSqlGenerate =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkSqlSplitter =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkJavaGenerate =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkBuild =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        buttonMessage =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        tabMessage =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        panelMessageOption =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        outputPanel =
            (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                            OutputPanel.class.getName());
        layoutMessageOption =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMessage =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        panelType = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        comboType = (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JComboBox.class.getName());
        layoutType =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
//        checkBuild = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    textRootPath.setEnabled(false);
        textRootPath.setEditable(false);
        textRootPath.setText("Output directory");
        buttonRootPath.setText("Set...");
    buttonRootPath.setToolTipText("");
    buttonRootPath.setText("Set...");
        panelRootPath.setLayout(layoutDomlFile);
        panelRootPath.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Output directory"));
        textDomlFile.setEnabled(true);
        textDomlFile.setEditable(true);
        textDomlFile.setText("");
        buttonDomlFile.setText("Set...");
        buttonMessage.setText("...");
        panelDomlFile.setLayout(layoutDomlFile);
        panelDomlFile.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Doml file"));
        //Dusan 12.02.2004.
        panelExtensions.setLayout(layoutExtensions);
        panelExtensions.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Extensions:"));
//        jTemplateTypeLabel.setText("Extensions :");
        jCheckBoxMultiDB.setText("Multi Database");
        jCheckBoxMultiDB.setSelected(false);
        jCheckBoxWebDocWF.setText("WebDocWF");
        jCheckBoxWebDocWF.setSelected(false);

        checkSqlGenerate.setText("SQL generate");
        checkSqlGenerate.setSelected(true);
        checkSqlSplitter.setText("SQL Splitter");
        checkSqlSplitter.setSelected(true);
        checkJavaGenerate.setText("Java generate");
        checkJavaGenerate.setSelected(true);
        checkBuild.setText("Invoke Dods Generator during Project Make/Rebuild");
        checkBuild.setSelected(false);
        panelMessageOption.setLayout(layoutMessageOption);
        this.setPreferredSize(new Dimension(483, 670));
    panelRootPath.add(textRootPath,
                            new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0,
                                                   GridBagConstraints.WEST,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));
        panelRootPath.add(buttonRootPath,
                            new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.NONE,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));
        panelDomlFile.add(textDomlFile,
                            new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0,
                                                   GridBagConstraints.WEST,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));
        panelDomlFile.add(buttonDomlFile,
                            new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.NONE,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));
        panelExtensions.add(jCheckBoxMultiDB,
                             new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));
        panelExtensions.add(jCheckBoxWebDocWF,
                             new GridBagConstraints(1, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 63, 0));
        panelMessage.setLayout(layoutMessage);
        panelMessage.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Dods messages"));
        this.setLayout(layoutMain);
        this.add(checkSqlGenerate,
                  new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 10, 2, 10), 0, 0));
        this.add(panelRootPath,
                  new GridBagConstraints(0, 0, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0));
        this.add(panelDomlFile,
                          new GridBagConstraints(0, 1, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 10, 15, 5), 0, 0));
        this.add(panelExtensions,
                      new GridBagConstraints(0, 3, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 5), 0, 0));
        this.add(checkSqlSplitter,
                  new GridBagConstraints(1, 4, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 10, 2, 10), 0, 0));
        this.add(checkJavaGenerate,
                   new GridBagConstraints(0, 5, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 10, 2, 10), 0, 0));
        this.add(panelType,
                  new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 2, 0));
    this.add(checkBuild, new GridBagConstraints(1, 5, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 10, 2, 10), 0, 0));
    this.add(panelMessage,      new GridBagConstraints(0, 2, 2, 1, 0.1, 0.0
            ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL, new Insets(-2, 8, 0, 2), 0, 71));
    panelMessage.add(tabMessage, BorderLayout.CENTER);
    tabMessage.add(outputPanel, "Messages");
    }

    private class CheckBoxListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == checkSqlGenerate) {
        	if (checkSqlGenerate.isSelected()) {
    		  checkSqlSplitter.setSelected(true);
    		  checkSqlSplitter.setEnabled(true);
	        }
        	else {
		  checkSqlSplitter.setSelected(false);
		  checkSqlSplitter.setEnabled(false);
  	        }
            }
            //Dusan 25.01.2003.
            if (source == checkSqlGenerate) {
        	if (!checkSqlGenerate.isSelected()) {
    		  checkJavaGenerate.setSelected(true);
	        }
            }
            if (source == checkJavaGenerate) {
        	if (!checkJavaGenerate.isSelected()) {
    		  checkSqlGenerate.setSelected(true);
    		  checkSqlSplitter.setSelected(true);
    		  checkSqlSplitter.setEnabled(true);
	        }
            }
      }
    }

    //
    private void chooseDomlFile() {
        File       choice = null;
        PathHandle path = null;
        ExtensionFilter filter = null;

        filter = new ExtensionFilter();
        filter.addExtension("doml");

/*        choice =
            SwingUtil.getFileChoice(this, textDomlFile.getText(),
              filter, "Select Doml file"); */
        choice =
            SwingUtil.getFileChoice(this, "c:\temp",
              filter, "Select Doml file");
        path = PathHandle.createPathHandle(choice);
        if (path.isFile()) {
            textDomlFile.setText(path.getPath());
            textDomlFile.setToolTipText(textDomlFile.getText());
        }
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonDomlFile) {
//Dusan 23.11.2002.
//                chooseDomlFile();
                browseForFile();
            } else if(source == buttonRootPath) {
                browseForDirectory();
            }
        }

    }

    /**
     * Method declaration
     *
     */
    private void browseForFile() {
        JFileChooser    chooser;
        ExtensionFilter filter;
        File            file;
        String          fileDir = new String();
        String          fileName = new String();

        filter = new ExtensionFilter();
        filter.addExtension(Constants.TYPE_DODS);
        filter.setDescriptionTitle(res.getString("filter_Doml_DescriptionTitle"));
        chooser = new JFileChooser();
        if (textDomlFile.getText().trim().length() > 0) {
            chooser.setCurrentDirectory(new File(textDomlFile.getText()));
        } else {
            PathHandle ph = null;
            chooser.setCurrentDirectory(new File(""));
        }
        chooser.setFileFilter(filter);
        chooser.setDialogTitle(res.getString("chooser_DODS_DialogTitle"));
        chooser.setApproveButtonText(res.getString("OK"));
        int v = chooser.showOpenDialog(this);

        this.requestFocus();
        buttonDomlFile.requestFocus();
        if (v == JFileChooser.APPROVE_OPTION) {
            if (chooser.getSelectedFile() == null
                    || (!chooser.getSelectedFile().isFile())) {
                textDomlFile.setText(new String());
            } else {
                fileDir = chooser.getCurrentDirectory().toString();
                fileDir = stripSlash(fileDir);
                fileName = chooser.getSelectedFile().getName();
                String strDomlFile = fileDir + File.separator + fileName;
                textDomlFile.setText(strDomlFile.replace('\\','/'));
           }
        }
        chooser.removeAll();
        chooser = null;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @return
     */
    private String stripSlash(String dir) {
        String stripped = new String(dir);

        if (dir != null) {
            if (dir.length() > 0) {
                if (dir.endsWith(File.separator)) {
                    stripped = dir.substring(0, dir.length() - 1);
                }
            }
        }
        return stripped;
    }

    //
    private boolean isValidDeployRoot(PathHandle ph) {
        boolean valid = false;

        if (!ph.isFile()) {
            if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    /**
     * Method declaration
     *
     */
    private void browseForDirectory() {
        JFileChooser    chooser;
        File            file;
        String          fileDir = new String();
        String          fileName = new String();

        chooser = new JFileChooser();
        if (textRootPath.getText().trim().length() > 0) {
            chooser.setCurrentDirectory(new File(textRootPath.getText()));
        } else {
            PathHandle ph = null;

            chooser.setCurrentDirectory(new File(""));
        }
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setFileFilter((javax.swing.filechooser.FileFilter) new DirectoryFilter());
        chooser.setDialogTitle(res.getString("chooser_RootPath_DialogTitle"));
        chooser.setApproveButtonText(res.getString("OK"));
        int v = chooser.showOpenDialog(this);

        this.requestFocus();
        buttonRootPath.requestFocus();
        if (v == JFileChooser.APPROVE_OPTION) {
            if (chooser.getSelectedFile() == null) {
                textRootPath.setText(new String());
            } else {
                fileDir = chooser.getCurrentDirectory().toString();
                fileDir = stripSlash(fileDir);
                fileName = chooser.getSelectedFile().getName();
                String strRootPath = fileDir + File.separator + fileName;
                textRootPath.setText(strRootPath.replace('\\','/'));
            }
        }
        chooser.removeAll();
        chooser = null;
    }

    }
