/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.importer;

// Kelp imports
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.ValidationUtil;
import org.enhydra.kelp.common.importer.ImportPaths;

// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.io.File;
import java.util.Properties;

/**
 * Panel for entering the default options: project name, package
 * and destination.
 *
 * @author Paul Mahar
 */
public class ImporterPanel extends JPanel {
    transient private ImportPaths paths;

    // strings not to be resourced
    private final String          WRITE_OPTIONS_EXCEPTION =
        "writeOptions() not implemented";  // nores
    private final String          VALIDATE_OPTIONS_EXCEPTION =
        "validateOptions() not implemented"; // nores
    private final String          PAGE_TITLE = "PAGE TITLE"; // nores
    private final String          INSTRUCTIONS = "INSTRUCTIONS"; // nores

    public void setPaths(ImportPaths p) {
        paths = p;
        readOptions();
    }

    public ImportPaths getPaths() {
        return paths;
    }

    public void activated() {}

    public void checkPage() throws ValidationException {
        writeOptions();
    }

    /**
     * Read project name, package and destination values from the
     * option set into the swing controls.
     *
     * @exception GeneratorException
     * Thrown if unable to update Swing controls with option set values.
     */
    public void readOptions() {}

    /**
     * Write project name, package and destination values from the
     * swing controls into the option set.
     *
     * @exception GeneratorException
     * Thrown if unable to update option set from Swing control values.
     */
    public void writeOptions() throws ValidationException {
        throw new ValidationException(WRITE_OPTIONS_EXCEPTION);
    }

    /**
     * Validate project name, package and destination values
     * in the swing controls.
     *
     * @exception ValidationException
     * Thrown if swing control values are not valid for the
     * current option set.
     */
    public void validateOptions() throws ValidationException {
        throw new ValidationException(VALIDATE_OPTIONS_EXCEPTION);
    }

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return PAGE_TITLE;
    }

    /**
     * Get the instructions for entering option values for the
     * current page.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append(INSTRUCTIONS);
        return buf.toString();
    }

}
