/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.xmlc;

// ToolBox imports
import org.enhydra.tool.common.DialogPanel;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.properties.XMLCParameterViewer;

// Standard imports
import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.event.*;
import java.io.File;
import java.awt.*;
import java.beans.*;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class XMLCOptionPanel extends JPanel {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private GridBagLayout       layoutMain = new GridBagLayout();
    private OtterNode           node = null;
    private JTextField          textFile = null;
    private JButton             buttonBrowse = null;
    private LocalButtonListener buttonListener = null;
    private TitledBorder        border = null;
    private JTextField          textCommandLine = null;
    private JLabel              labelCommandLine = null;
    private JLabel              labelFile = null;
    private JButton             buttonClear = null;
    private JButton             buttonTool = null;

    /**
     * Constructor declaration
     *
     */
    public XMLCOptionPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * Method declaration
     *
     *
     * @param e
     */
    public void setEnabled(boolean b) {
        super.setEnabled(b);
        textCommandLine.setEnabled(b);
        textFile.setEnabled(b);
        buttonTool.setEnabled(b);
        buttonBrowse.setEnabled(b);
        buttonClear.setEnabled(b);
        if (b) {
            textCommandLine.setBackground(SystemColor.info);
            textFile.setBackground(SystemColor.info);
        } else {
            textCommandLine.setBackground(SystemColor.text);
            textFile.setBackground(SystemColor.text);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterNode getNode() {
        return node;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setNode(OtterNode n) {
        node = n;
    }

    /**
     * Method declaration
     *
     */
    public void readProperties() {
        String filename = null;
        String parameters = null;

        if (node == null) {
            buttonBrowse.setEnabled(false);
        } else {
            buttonBrowse.setEnabled(true);
            filename = node.getXMLCOptionFilePath();
            parameters = node.getXMLCParameters();
        }
        if (filename != null) {
            File file = new File(filename);

            if (file.isFile()) {
                textFile.setText(filename);
            }
        }
        if (parameters != null) {
            textCommandLine.setText(parameters);
        }
    }

    /**
     * Method declaration
     *
     */
    public void writeProperties() {
        String parameters = textCommandLine.getText().trim();

        if (node != null) {
            node.setXMLCParameters(parameters);
            node.setXMLCOptionFilePath(getValidFilename());
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    private String getValidFilename() {
        PathHandle path = null;
        String     returnPath = new String();

        path = PathHandle.createPathHandle(textFile.getText());
        if (path.isFile() && path.hasExtension(Constants.TYPE_XMLC)) {
            returnPath = path.getPath();
            textFile.setText(path.getPath());
        } else {
            textFile.setText(new String());
        }
        return returnPath;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    private boolean isValidFilename() {
        return (getValidFilename().length() > 0);
    }

    /**
     * Method declaration
     *
     */
    private void browseForFile() {
        JFileChooser    chooser;
        ExtensionFilter filter;
        File            file;
        String          fileDir = new String();
        String          fileName = new String();

        filter = new ExtensionFilter();
        filter.addExtension(Constants.TYPE_XMLC);
        filter.setDescriptionTitle(res.getString("filter_DescriptionTitle"));
        chooser = new JFileChooser();
        if (textFile.getText().trim().length() > 0) {
            chooser.setCurrentDirectory(new File(textFile.getText()));
        } else {
            PathHandle ph = null;

            if (node instanceof OtterFileNode) {
               OtterFileNode fileNode = (OtterFileNode) node;
                ph = PathHandle.createPathHandle(fileNode.getFilePath());
                if (ph.isEmpty()) {
                  ph = null;
                } else {
                  ph = ph.getParent();
                }
            }
            if (ph == null) {
                ph = PathHandle.createPathHandle(node.getProject().getRootPath());
            }
            chooser.setCurrentDirectory(ph.getFile());
        }
        chooser.setFileFilter(filter);
        chooser.setDialogTitle(res.getString("chooser_DialogTitle"));
        chooser.setApproveButtonText(res.getString("OK"));
        int v = chooser.showOpenDialog(this);

        this.requestFocus();
        buttonBrowse.requestFocus();
        if (v == JFileChooser.APPROVE_OPTION) {
            if (chooser.getSelectedFile() == null
                    || (!chooser.getSelectedFile().isFile())) {
                textFile.setText(new String());
            } else {
                fileDir = chooser.getCurrentDirectory().toString();
                fileDir = stripSlash(fileDir);
                fileName = chooser.getSelectedFile().getName();
                textFile.setText(fileDir + File.separator + fileName);
            }
        }
        chooser.removeAll();
        chooser = null;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @return
     */
    private String stripSlash(String dir) {
        String stripped = new String(dir);

        if (dir != null) {
            if (dir.length() > 0) {
                if (dir.endsWith(File.separator)) {
                    stripped = dir.substring(0, dir.length() - 1);
                }
            }
        }
        return stripped;
    }

    private void editParameters() {
        String              params = new String();
        XMLCParameterViewer viewer = null;

        params = textCommandLine.getText();
        viewer = new XMLCParameterViewer();
        viewer.setParameters(params);
        viewer.setOwner((Dialog) getTopLevelAncestor());
        if (viewer.showDialog() == DialogPanel.OK_OPTION) {
            params = viewer.getParameters();
            textCommandLine.setText(params);
        }
    }

    /**
     * Method declaration
     *
     */
    private void pmInit() {
        textCommandLine.setText(new String());
        buttonListener = new LocalButtonListener();
        buttonBrowse.addActionListener(buttonListener);
        buttonClear.addActionListener(buttonListener);
        buttonTool.addActionListener(buttonListener);
        setEnabled(true);
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit() throws Exception {
        buttonBrowse =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        textFile = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        border =
            BorderFactory.createTitledBorder(res.getString("XMLC_Options"));
        textCommandLine =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        labelCommandLine =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        labelFile = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        buttonClear = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonTool = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        textFile.setMinimumSize(new Dimension(100, 21));
        textFile.setPreferredSize(new Dimension(100, 21));
        textFile.setEditable(false);
        buttonBrowse.setText(res.getString("Browse"));
        labelCommandLine.setText(res.getString("labelCommandLine_Text"));
        labelFile.setText(res.getString("labelFile_Text"));
        buttonClear.setText(res.getString("Clear"));
        buttonTool.setText("..."); // nores
        this.setLayout(layoutMain);
        this.setBorder(border);
        this.add(textFile,
                 new GridBagConstraints(0, 4, 1, 1, 1.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(1, 2, 2, 2), 100, 0));
        this.add(buttonBrowse,
                 new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(1, 2, 2, 2), 0, 0));
        this.add(labelCommandLine,
                 new GridBagConstraints(0, 1, 4, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 2, 1, 2), 0, 0));
        this.add(textCommandLine,
                 new GridBagConstraints(0, 2, 3, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(1, 2, 2, 2), 0, 0));
        this.add(labelFile,
                 new GridBagConstraints(0, 3, 4, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 2, 1, 2), 0, 0));
        this.add(buttonClear,
                 new GridBagConstraints(2, 4, 2, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(1, 2, 2, 2), 0, 0));
        this.add(buttonTool,
                 new GridBagConstraints(3, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(1, 2, 2, 2), 0, 0));
    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonBrowse) {
                browseForFile();
            } else if (source == buttonClear) {
                textFile.setText(new String());
            } else if (source == buttonTool) {
                editParameters();
            }
        }

    }
}
