
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// ToolBox
import org.enhydra.tool.archive.wizard.ArchiveType;
import org.enhydra.tool.archive.wizard.E3Type;
import org.enhydra.tool.archive.wizard.EarType;
import org.enhydra.tool.archive.wizard.EjbType;
import org.enhydra.tool.archive.wizard.WarType;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

//
public class PropertyWriter implements PropertyKeys, ManifestKeys {

    //
    private final String[] types = {
        E3Type.EXT, EarType.EXT, EjbType.EXT, WarType.EXT
    };
    private File           file = null;
    private Properties     props = new Properties();
    private int            count = 0;

    public PropertyWriter() {}

    public void write(JarPlan[] plans, File out) {
        FileOutputStream     fileStream = null;
        BufferedOutputStream bufStream = null;

        file = out;
        write(plans);
        try {
            fileStream = new FileOutputStream(file);
            bufStream = new BufferedOutputStream(fileStream);
            props.store(bufStream, "Kelp Archive Properties");
            fileStream.flush();
            bufStream.flush();
            bufStream.close();
            fileStream.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace(System.err);
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }

    public Properties write(JarPlan[] plans) {
        for (int i = 0; i < plans.length; i++) {
            write(plans[i]);
        }
        return props;
    }

    public Properties write(String[] jars) {

        // TODO, add parameter to save file paths relative to
        // TODO, move relative stuff from Kelp to common.
        for (int i = 0; i < jars.length; i++) {
            write(jars[i]);
        }
        return props;
    }

    public Properties write(JarPlan plan) {
        String value = null;

        value = plan.getArchivePath();
        props.setProperty(PATH + count, pathValue(value));
        if (plan instanceof WarPlan) {
            writeWarProps((WarPlan) plan);
        } else if (plan instanceof EarPlan) {
            writeEarProps((EarPlan) plan);
        } else if (plan instanceof EjbPlan) {
            writeEjbProps((EjbPlan) plan);
        } else {
            writeJarProps(plan);
        }
        count++;
        return props;
    }

    private void write(String archive) {
        ArchiveType type = null;
        PathHandle  ph = null;
        JarPlan     plan = null;

        ph = PathHandle.createPathHandle(archive);
        type = getType(ph.getPath());
        if (type != null) {
            type.readArchive(ph.getPath());
            try {
                plan = type.getPlan();
            } catch (ArchiveException e) {
                e.printStackTrace(System.err);
            }
            write(plan);
        }
    }

    private void writeWarProps(WarPlan plan) {
        String[] files = new String[0];
        String   root = null;
        String   dd = null;

        writeJarProps(plan);

        // content
        files = plan.getContentFiles();
        root = plan.getContentRoot();
        props.setProperty(CONTENT_ROOT_PROP + count, pathValue(root));
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                props.setProperty(CONTENT + count + "." + i,
                                  pathValue(files[i]));
            }
        }

        // libs
        files = plan.getLibFiles();
        for (int i = 0; i < files.length; i++) {
            props.setProperty(LIBRARY_PROP + count + "." + i,
                              pathValue(files[i]));
        }
    }

    private void writeEarProps(EarPlan plan) {
        String[]         files = new String[0];
        Descriptor[]     dd = new Descriptor[0];
        WebApplication[] webApps = new WebApplication[0];
        Boolean          b = null;

        // app name
        props.setProperty(APP_NAME_PROP + count, plan.getAppName());

        // descriptiion
        props.setProperty(APP_DESC_PROP + count, plan.getDescription());

        // alt-dd
        b = new Boolean(plan.isAlt());
        props.setProperty(ALT_DD_PROP + count, b.toString());

        // copy-client
        b = new Boolean(plan.isCopyClient());
        props.setProperty(COPY_CLIENT_PROP + count, b.toString());

        // enterprise mods
        files = plan.getEnterpriseModules();
        for (int i = 0; i < files.length; i++) {
            props.setProperty(ENTERPRISE_MODULE_PROP + count + "." + i,
                              pathValue(files[i]));
        }

        // web apps
        webApps = plan.getWebApplications();
        for (int i = 0; i < webApps.length; i++) {
            props.setProperty(WEBAPP_MODULE_PROP + count + "." + i,
                              pathValue(webApps[i].getArchive()));
            props.setProperty(WEBAPP_CONTEXT_PROP + count + "." + i,
                              webApps[i].getContextRoot());
        }

        // meta-dd
        dd = plan.getDescriptors();
        for (int i = 0; i < dd.length; i++) {
            props.setProperty(META_DD_PROP + count + "." + i,
                              pathValue(dd[i].getPath()));
        }

        // libs
        files = plan.getLibFiles();
        for (int i = 0; i < files.length; i++) {
            props.setProperty(LIBRARY_PROP + count + "." + i,
                              pathValue(files[i]));
        }

        // TODO: root-context
    }

    private void writeEjbProps(EjbPlan plan) {
        Boolean b = null;

        writeJarProps(plan);

        // copy-client
        b = new Boolean(plan.isCreateClient());
        props.setProperty(CREATE_CLIENT_PROP + count, b.toString());
    }

    private void writeJarProps(JarPlan plan) {
        String[]     files = new String[0];
        Descriptor[] dd = new Descriptor[0];
        String       root = null;

        // classes
        root = plan.getClassRoot();
        files = plan.getClassFiles();
        props.setProperty(CLASS_ROOT_PROP + count, pathValue(root));
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                props.setProperty(CLASS + count + "." + i,
                                  pathValue(files[i]));
            }
        }

        // deployment descriptors
        dd = plan.getDescriptors();
        for (int i = 0; i < dd.length; i++) {
            props.setProperty(dd[i].getType() + "." + count,
                              pathValue(dd[i].getPath()));
        }
    }

    private ArchiveType getType(String archive) {
        PathHandle  ph = null;
        JarFile     jar = null;
        Manifest    manifest = null;
        ArchiveType type = null;
        String      value = null;

        ph = PathHandle.createPathHandle(archive);
        if (ph.isFile() && ph.getFile().canRead() && ph.hasExtension(types)) {
            try {
                jar = new JarFile(ph.getFile());
                manifest = jar.getManifest();
                value = manifest.getMainAttributes().getValue(KELP_VERSION);
                if (value == null) {
                    System.err.println("Kelp properties not found in manifest of: "
                                       + ph.getPath());
                } else if (ph.hasExtension(WarType.EXT)) {
                    type = new WarType();
                } else if (ph.hasExtension(EarType.EXT)) {
                    type = new EarType();
                } else {
                    value =
                        manifest.getMainAttributes().getValue(CREATE_CLIENT);
                    if (value == null) {
                        type = new E3Type();
                    } else {
                        type = new EjbType();
                    }
                }
            } catch (IOException e) {
                e.printStackTrace(System.err);
            }
        }
        return type;
    }

    private String pathValue(String in) {
        String out = null;

        if (file == null) {
            out = in;
        } else {
            PathHandle propPh = null;
            PathHandle inPh = null;

            propPh = PathHandle.createPathHandle(file);
            inPh = PathHandle.createPathHandle(in);
            out = propPh.getRelativePath(inPh);
        }
        return out;
    }

}
