
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;

/**
 * The ValidationException can be thrown when validating option values.
 */
public class ValidationException extends GeneratorException {
    transient private String validationMsg = new String();

    /**
     * Create a GeneratorException with a specified error message.
     *
     * @param message
     * Message providing information on the nature of the exception.
     */
    public ValidationException(String message) {
        super(message);
        validationMsg = message;
    }

    /**
     * Create a ValidationException linked to a parent exception.
     *
     * @param chain
     * Parent exception to include in stack trace.
     * @param message
     * Message providing information on the nature of the exception.
     */
    public ValidationException(Throwable chain, String message) {
        super(chain, message);
        validationMsg = message;
    }

    /**
     * Get the message of this exception without any other chain
     * member messages.
     *
     * @return
     * A single exception message.
     */
    public String getValidationMessage() {
        return validationMsg;
    }

}
