/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.build;

// JBuilder imports
import com.borland.primetime.build.BuildTask;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.TextFileNode;
import com.borland.jbuilder.node.JBProject;

// Kelp imports

// Standard imports
import java.util.Vector;
import java.util.ResourceBundle;
import java.io.File;
import org.enhydra.kelp.ant.node.AntProject;
import java.io.*;
import com.borland.primetime.vfs.Url;
import org.enhydra.kelp.jbuilder.node.NodeUtil;
import com.borland.primetime.node.LightweightNode;

/**
 * BuildTask implementation for calling Clean.
 *
 */
public class CleanBuildTask extends BuildTask {

    public static final String SHARED_KEY = new String("Clean_TASK");

    /**
     * Constructor declaration
     *
     */
    public CleanBuildTask() {}

    /**
     * Generate the DOM class using Clean.
     */
    public boolean build(BuildProcess buildProcess) {
      String prjPath = buildProcess.getProject().getProjectPath().getFullName();
      try {
        AntProject antProject = new AntProject(prjPath);
        buildProcess.fireBuildStatus("Kelp cleaning ...", true);
        String enhydraDir = antProject.getProperty(AntProject.ENHYDRA_DIR);
        String execLine = enhydraDir + "/bin/ant";
        if (System.getProperty("os.name").toLowerCase().startsWith("windows")) {
          execLine += ".bat";
        }
        Process process = Runtime.getRuntime().exec(execLine + " clean", null,
                                                    new File(prjPath));
        InputStream inputStream = process.getInputStream();
        InputStream errorStream = process.getErrorStream();
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
        BufferedReader errorBufferedReader = new BufferedReader(new InputStreamReader(errorStream));
        File logFile = null;
        FileWriter logFileWriter = null;
        BufferedWriter logBuffer = null;
        PrintWriter logPrint = null;

        String s = null;
        String er = null;
        while(((s = bufferedReader.readLine()) != null) || ((er = errorBufferedReader.readLine()) != null)){
          if(s != null){
            System.out.println(s);
            s = null;
          }
          if(er != null){
            System.out.println(s);
            er = null; //must reset becouse second condition is not executed if firs condition is true
          }
        }

        Url url = new Url(new File(antProject.getProperty(AntProject.OUTPUT_DIR)));
        try {
          LightweightNode nodes[] = buildProcess.getProject()
              .findNodes(url.getName());
          for (int i = 0; i < nodes.length; i++) {
            nodes[i].setParent(null);
          }
        }
        catch (Exception ex) {
          ex.printStackTrace();
        }
      }
      catch (Exception ex) {
        ex.printStackTrace();
      }
      return true;
    }
}
