/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.build;

import com.borland.primetime.build.BuildTask;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.TextFileNode;
import com.borland.jbuilder.node.JBProject;

import java.util.Vector;
import java.util.ResourceBundle;
import java.io.File;
import org.enhydra.kelp.ant.node.AntProject;
import java.io.*;

/**
 * BuildTask implementation for calling DODS.
 *
 */
public class DODSBuildTask extends BuildTask {

    public static final String SHARED_KEY = new String("DODS_TASK");
    private BuildProcess       buildProcess;

    /**
     * Constructor declaration
     *
     */
    public DODSBuildTask() {}

    /**
     * Generate the DOM class using DODS.
     */
    public boolean build(BuildProcess buildProcess) {
      this.buildProcess = buildProcess;
      String prjPath = buildProcess.getProject().getProjectPath().getFullName();
      try {
        AntProject antProject = new AntProject(prjPath);
        buildProcess.fireBuildStatus("DODS Generating...", true);
        String enhydraDir = antProject.getProperty(AntProject.ENHYDRA_DIR);
        String execLine = enhydraDir + "/bin/ant";
        if (System.getProperty("os.name").toLowerCase().startsWith("windows")) {
          execLine += ".bat";
        }
        Process process = Runtime.getRuntime().exec(execLine + " dods", null,
                                                    new File(prjPath));
        boolean finished = false;
        // another build processes need to wait this batch to finish
        while (!finished) {
          finished = true;
          try {
            process.exitValue();
          }
          catch (Exception ex) {
            finished = false;
          }
        }

      }
      catch (Exception ex) {
        ex.printStackTrace();
        return false;
      }
      return true;
    }

}
