package org.enhydra.kelp.jdev;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;

import java.net.URL;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import oracle.ide.ContextMenu;
import oracle.ide.IdeAction;
import oracle.ide.MainWindow;
import oracle.ide.addin.Addin;
import oracle.ide.addin.Context;
import oracle.ide.addin.ContextMenuListener;
import oracle.ide.addin.Controller;
import oracle.ide.model.Element;
import oracle.ide.model.TextNode;

import org.enhydra.kelp.ant.deployer.AntDeployTool;
import org.enhydra.kelp.ant.dods.KelpDODSGenerator;
import org.enhydra.kelp.ant.xmlc.AntXMLCTool;
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.kelp.ant.node.AntProject;
import oracle.ide.Ide;
import oracle.ide.model.Project;
import oracle.ide.model.Workspace;
import oracle.jdeveloper.model.JProject;
import oracle.ideimpl.explorer.ExplorerNode;
import oracle.ideimpl.navigator.NavigatorWindowImpl;

public class EnhydraTools implements Addin, Controller, ContextMenuListener 
{
  private static JMenuItem contextMenuItem = null;

  public static final int XMLC_CMD_ID = Ide.newCmd("org.enhydra.kelp.jdev.EnhydraTools.XMLC_CMD_ID");
  public static final int DEPLOYER_CMD_ID = Ide.newCmd("org.enhydra.kelp.jdev.EnhydraTools.DEPLOYER_CMD_ID");
  public static final int DODS_CMD_ID = Ide.newCmd("org.enhydra.kelp.jdev.EnhydraTools.DODS_CMD_ID");
  public static final int ANT_REBUILD_CMD_ID = Ide.newCmd("org.enhydra.kelp.jdev.EnhydraTools.ANT_REBUILD_CMD_ID");

  private final static String MENU_ICON = "/media/menu_16x16.gif";
  

  private JMenuItem menuItem = null;
  public EnhydraTools()
  {
  }

  public void initialize()
  {
    // Do not do major initialization here, it can slow down the startup of JDeveloper
    JMenu enhydraGroup = Ide.getMenubar().createSubMenu("Enhydra Tools", new Integer('E'));
    JMenuItem menuItemXMLC = createMenu_XMLC_CMD_ID(MENU_ICON, "XML Compiler", 88); // X
    JMenuItem menuItemDeployer = createMenu_DEPLOYER_CMD_ID(null, "Deployer", 68); // D
    JMenuItem menuItemDODS = createMenu_DODS_CMD_ID(MENU_ICON, "DODS", 79); // O
    enhydraGroup.add(menuItemXMLC);
    enhydraGroup.add(menuItemDeployer);
    enhydraGroup.add(menuItemDODS);
    JMenu mainMenu = MainWindow.Tools;
    mainMenu.add(enhydraGroup);
    Ide.getNavigatorManager().addContextMenuListener(this, null);
  }

  public void shutdown()
  {
    // Do any necessary cleanup here
  }

  public float version()
  {
    // return the version of your extension
    return 1.0f;
  }

  public float ideVersion()
  {
    return Ide.IDE_VERSION;
  }

  public boolean canShutdown()
  {
    return true;
  }

  private Icon loadIcon(String iconName)
  {
    URL url = getClass().getResource(iconName);
    Icon icon = null;
    if (url != null)
    {
      icon = new ImageIcon(url);
    }
    return icon;
  }

  private JMenuItem createMenu_XMLC_CMD_ID(String iconName, String menuLabel, int mnemonic)
  {
    // String class name which extends oracle.ide.addin.AbstractCommand or null
    String cmdClass = null;
    // Category of the keymap to which this entry belongs
    String category = null;
    // Icon for the menu item
    Icon icon = null;
    if (iconName != null)
      icon = loadIcon(iconName);
    // Data to be associated with the Action
    Object data = null;
    // Should the action be enabled by default
    boolean enabled = true;

    IdeAction action;
    // Create the IdeAction
    action = IdeAction.get(XMLC_CMD_ID, cmdClass, menuLabel, category, new Integer(mnemonic), icon, data, enabled);
    // Set the default controller for this action
    action.setController(this);
    // Create MenuItem
    JMenuItem menuItem = Ide.getMenubar().createMenuItem(action);
    
    return menuItem;
  }

  private JMenuItem createMenu_DEPLOYER_CMD_ID(String iconName, String menuLabel, int mnemonic)
  {
    // String class name which extends oracle.ide.addin.AbstractCommand or null
    String cmdClass = null;
    // Category of the keymap to which this entry belongs
    String category = null;
    // Icon for the menu item
    Icon icon = null;
    if (iconName != null)
      icon = loadIcon(iconName);
    // Data to be associated with the Action
    Object data = null;
    // Should the action be enabled by default
    boolean enabled = true;

    IdeAction action;
    // Create the IdeAction
    action = IdeAction.get(DEPLOYER_CMD_ID, cmdClass, menuLabel, category, new Integer(mnemonic), icon, data, enabled);
    // Set the default controller for this action
    action.setController(this);
    // Create MenuItem
    JMenuItem menuItem = Ide.getMenubar().createMenuItem(action);
    
    return menuItem;
  }

  private JMenuItem createMenu_DODS_CMD_ID(String iconName, String menuLabel, int mnemonic)
  {
    // String class name which extends oracle.ide.addin.AbstractCommand or null
    String cmdClass = null;
    // Category of the keymap to which this entry belongs
    String category = null;
    // Icon for the menu item
    Icon icon = null;
    if (iconName != null)
      icon = loadIcon(iconName);
    // Data to be associated with the Action
    Object data = null;
    // Should the action be enabled by default
    boolean enabled = true;

    IdeAction action;
    // Create the IdeAction
    action = IdeAction.get(DODS_CMD_ID, cmdClass, menuLabel, category, new Integer(mnemonic), icon, data, enabled);
    // Set the default controller for this action
    action.setController(this);
    // Create MenuItem
    JMenuItem menuItem = Ide.getMenubar().createMenuItem(action);
    
    return menuItem;
  }

  public Controller supervisor()
  {
    return null;
  }

  public boolean handleEvent(IdeAction action, Context context)
  {
    final int cmdId = action.getCommandId();
    String prjPath = Ide.getActiveWorkspace().getActiveProjectURL().getFile();
    prjPath = prjPath.substring(0, prjPath.lastIndexOf("/"));
    boolean refresh = false;
    if (cmdId == XMLC_CMD_ID) 
    {
      AntXMLCTool.main(new String[]{prjPath});
      refresh = true;
    }
    else if (cmdId == DEPLOYER_CMD_ID) 
    {
      AntDeployTool.main(new String[]{prjPath});
      refresh = true;
    }
    else if (cmdId == DODS_CMD_ID)
    {
      try {
        AntProject antProject = new AntProject(prjPath);
        String enhydraDir = antProject.getProperty(AntProject.ENHYDRA_DIR);
        File dodsDirFile =  new File(enhydraDir, "dods");
        System.setProperty("DODS_HOME", dodsDirFile.getAbsolutePath());
        KelpDODSGenerator.main(new String[]{prjPath});
        refresh = true;
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    if (refresh)
    {
      Workspace ws = Ide.getActiveWorkspace();
      JProject activeProj = (JProject)Ide.getActiveProject();
      refreshNavigator(ws, activeProj);
      return true;
    } 
    else
      return false;
  }

  public boolean update(IdeAction action, Context context)
  {
    final int cmdId = action.getCommandId();
    if (cmdId == org.enhydra.kelp.jdev.EnhydraTools.XMLC_CMD_ID)
    {
      action.setEnabled(is_XMLC_CMD_ID_Available(context));
      return true;
    }
    return false;
  }

  private boolean is_XMLC_CMD_ID_Available(Context context)
  {
    if (context == null)
    {
      return false;
    }
    Element element = context.getElement();
    if (element == null)
    {
      return false;
    }
    if (element instanceof Project)
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  public void checkCommands(Context action, Controller controller)
  {
    // If your command is on the main menu and has an accelerator you need to call update from here
    // If your command is on the toolbar then you also must call update from here
  }

  private JMenuItem getContextMenuItem()
  {
    if (contextMenuItem == null)
    {
      contextMenuItem = createMenu_XMLC_CMD_ID(MENU_ICON, "XMLC", 88);
    }
    return contextMenuItem;
  }

  public void poppingUp(ContextMenu popup)
  {
    // The getContextMenuItem function will lazily instantiate the menu item
    popup.add(getContextMenuItem());
  }

  public void poppingDown(ContextMenu popup)
  {
  }

  public boolean handleDefaultAction(Context context)
  {
    // Return false since this is not the default action
    return false;
  }

  private void refreshNavigator(Workspace ws, JProject enhydraProj) 
  {
    NavigatorWindowImpl navWin = (NavigatorWindowImpl)
            Ide.getNavigatorManager().getSystemNavigator();
    navWin.show();
    ExplorerNode eNode = (ExplorerNode)navWin.getTreeExplorer().getRoot();
    eNode = (ExplorerNode)navWin.findTNode(ws, eNode);
    navWin.refresh(eNode);
    eNode = (ExplorerNode)navWin.findTNode(enhydraProj, eNode);
    navWin.expand(eNode, true);
    navWin.show();
  }

}