
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

//
import org.enhydra.tool.archive.wizard.ArchiveWizard;
import org.enhydra.tool.common.PathHandle;

// Standard imports
import java.io.File;
import java.util.Properties;

//
public class ArchiveTool {

    // not to be resourced
    private final String NULL_PLAN = "Archive plan is null";    // nores

    public ArchiveTool() {}

    public File buildArchive(EarPlan plan) throws ArchiveException {
        EarBuilder builder = null;
        File       file = null;

        if (plan == null) {
            throw new ArchiveException(NULL_PLAN);
        }
        builder = new EarBuilder();
        builder.setPlan(plan);
        file = builder.buildArchive();
        return file;
    }

    public File buildArchive(EjbPlan plan) throws ArchiveException {
        EjbBuilder builder = null;
        File       file = null;

        if (plan == null) {
            throw new ArchiveException(NULL_PLAN);
        }
        builder = new EjbBuilder();
        builder.setPlan(plan);
        file = builder.buildArchive();
        return file;
    }

    public File buildArchive(WarPlan plan) throws ArchiveException {
        WarBuilder builder = null;
        File       file = null;

        if (plan == null) {
            throw new ArchiveException(NULL_PLAN);
        }
        builder = new WarBuilder();
        builder.setPlan(plan);
        file = builder.buildArchive();
        return file;
    }

    public File buildArchive(JarPlan plan) throws ArchiveException {
        JarBuilder builder = null;
        File       file = null;

System.out.println("ArchiveTool.BuildArchive");
        if (plan instanceof EjbPlan) {
System.out.println("EjbPlan");
            file = buildArchive((EjbPlan) plan);
        } else if (plan instanceof EarPlan) {
System.out.println("EarPlan");
            file = buildArchive((EarPlan) plan);
        } else if (plan instanceof WarPlan) {
System.out.println("WarPlan");
            file = buildArchive((WarPlan) plan);
        } else {
System.out.println("NULL_PLAN");
            if (plan == null) {
                throw new ArchiveException(NULL_PLAN);
            }
            builder = new JarBuilder();
            builder.setPlan(plan);
            file = builder.buildArchive();
        }
        return file;
    }

    public static JarPlan runWizard(int i) {
        ArchiveWizard handler = null;
        JarPlan plan  = null;

        handler = new ArchiveWizard();
        handler.showDialog(null);
        plan = handler.getPlan();
        handler = null;
        return plan;

    }
    public static String runWizard() {
        JarPlan plan = null;
        String newArchive = null;
        plan = ArchiveTool.runWizard(0);
        if (plan != null) {
            newArchive =
                PathHandle.createPathString(plan.getArchivePath());
        }
        return newArchive;
    }

}
