
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// ToolBox imports
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.archive.xml.WarDescriptorHandler;

// Standard imports
import java.io.IOException;
import java.io.File;
import java.util.ResourceBundle;
import java.util.jar.Manifest;
import java.util.jar.Attributes;

//
public class WarBuilder extends JarBuilder {

    // not to be resourced
    private final String       WEB_INF = "WEB-INF";            // nores
    private final String       CLASSES = "classes";            // nores

    //
    public WarBuilder() {
        super();
    }

    public WarPlan getWarPlan() {
        WarPlan wp = null;

        if (getPlan() instanceof WarPlan) {
            wp = (WarPlan) getPlan();
        }
        return wp;
    }

    // override JarBuilder
    public File buildArchive() throws ArchiveException {
        File         war = null;
        String       root = null;
        File[]       files = new File[0];
        Descriptor[] dd = new Descriptor[0];

        war = openJarStream(getWarPlan().getArchivePath());
        root = getWarPlan().getContentRoot();
        dd = getWarPlan().getDescriptors();
        files = getWarPlan().getContentFileArray();
        addWarClasses();
        addLibraries();
        addContent();
        for (int i = 0; i < dd.length; i++) {
            addDescriptor(dd[i], root, files);
        }
        closeJarStream();
        return war;
    }

    protected Manifest buildManifest(Manifest m) {
        Manifest manifest = super.buildManifest(m);

        manifest.getMainAttributes().put(new Attributes.Name(CONTENT_ROOT),
                                         getWarPlan().getContentRoot());
        return manifest;
    }

    private void addContent() throws ArchiveException {
        File[] files = getWarPlan().getContentFileArray();

        for (int i = 0; i < files.length; i++) {
            addContentFile(files[i]);
        }
    }

    private void addContentFile(File source) throws ArchiveException {
        StringBuffer entryPath = new StringBuffer();
        String       contentRoot = null;
        String       relative = null;

        contentRoot = getWarPlan().getContentRoot();
        relative = source.getAbsolutePath().substring(contentRoot.length()
                + 1);
        relative = relative.replace('\\', '/');
        entryPath.append(relative);
        try {
            createEntry(source, entryPath.toString());
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       ResUtil.format(res.getString("WarBuilder_Unable_to"),
                                                      source));
        }
    }

    private void addLibraries() throws ArchiveException {
        File[] files = getWarPlan().getLibFileArray();

        for (int i = 0; i < files.length; i++) {
            addLibFile(files[i]);
        }
    }

    private void addLibFile(File source) throws ArchiveException {
        StringBuffer entryPath = new StringBuffer();

        //
        entryPath.append("WEB-INF");
        entryPath.append('/');
        entryPath.append("lib");
        entryPath.append('/');
        entryPath.append(source.getName());

        //
        try {
            createEntry(source, entryPath.toString());
        } catch (IOException e) {
            String exMess = res.getString("WarBuilder_Unable_to");

            exMess = ResUtil.format(exMess, source);
            throw new ArchiveException(e, exMess);
        }
    }

    private void addWarClasses() throws ArchiveException {
        File[] files = getWarPlan().getClassFileArray();

        for (int i = 0; i < files.length; i++) {
            addWarClassesFile(files[i]);
        }
    }

    private void addWarClassesFile(File source) throws ArchiveException {
        StringBuffer entryPath = new StringBuffer();
        String       classRoot = null;
        String       relative = null;

        //
        classRoot = getWarPlan().getClassRoot();
        relative = source.getAbsolutePath().substring(classRoot.length() + 1);
        relative = relative.replace('\\', '/');

        //
        entryPath.append(WEB_INF);
        entryPath.append('/');
        entryPath.append(CLASSES);
        entryPath.append('/');
        entryPath.append(relative);

        //
        try {
            createEntry(source, entryPath.toString());
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       ResUtil.format(res.getString("WarBuilder_Unable_to1"),
                                                      source));
        }
    }

}
