
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.FilenameFilter;
import org.enhydra.tool.common.SwingUtil;

// JDK
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import java.beans.Beans;

//
public class FilePanel extends RefPanel {
    private JPanel              panelDescript = null;
    private GridBagLayout       layoutMain = null;
    private JPanel              panelFile = null;
    private JTextField          textFile = null;
    private JButton             buttonFile = null;
    private GridBagLayout       layoutFile = null;
    private GridBagLayout       layoutDescript = null;
    private JButton             buttonDD = null;
    private LocalButtonListener buttonListener = null;
    private JScrollPane         scroll = null;
    private JTable              tableDD = null;
    private String              extension = "jar";
    private String              title = "Archive";
    private String[]            defaultDD = new String[0];

    public FilePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    protected void setExtension(String s) {
        extension = s;
    }

    protected String getExtension() {
        return extension;
    }

    protected void setTitle(String s) {
        title = s;
    }

    protected String getTitle() {
        return title;
    }

    protected String[] getDefaultDD() {
        return defaultDD;
    }

    protected void setDefaultDD(String[] dd) {
        defaultDD = dd;
    }

    protected void validatePlan() throws ArchiveException {
        int count = -1;

        if (getArchivePath().trim().length() == 0) {
            throw new ArchiveException("Archive file name required");
        } else {
            PathHandle path = null;

            path = PathHandle.createPathHandle(getArchivePath());
            if (!path.hasExtension(getExtension())) {
                throw new ArchiveException("Invalid archive extension: "
                                           + path.getExtension()
                                           + "\n Expecting: "
                                           + getExtension());
            }
        }
        count = getDDTableModel().getRowCount();
        for (int i = 0; i < count; i++) {
            Descriptor row = null;

            row = getDDTableModel().getDescriptor(i);
            if (row.isRequired()) {
                if (row.getPath().trim().length() == 0) {
                    throw new ArchiveException(row.getType()
                                               + " descriptor required");
                } else {
                    File file = null;

                    file = new File(row.getPath());
                    if (!file.isFile()) {
                        throw new ArchiveException("Invalid descriptor: "
                                                   + file.getAbsolutePath());
                    }
                }
            }
        }
    }

    protected String getArchivePath() {
        return textFile.getText();
    }

    protected void setArchivePath(String path) {
        PathHandle handle = PathHandle.createPathHandle(path);

        textFile.setText(handle.getPath());
        if (getWizard() != null) {
            for (int i = 0; i < 3; i++) {
                if (handle.getParent() == null) {
                    break;
                } else {
                    handle = handle.getParent();
                }
            }
            if (handle.isDirectory()) {
                setWorkingRoot(handle.getPath());
            }
        }
    }

    protected DDTableModel getDDTableModel() {
        return (DDTableModel) tableDD.getModel();
    }

    protected JTable getDDTable() {
        return tableDD;
    }

    protected void preparePanel() {
        panelFile.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(new Color(187, 218, 252), new Color(91, 107, 123)),
                                             getTitle()));
        setArchivePath(getArchivePath());
        if (!getDDTable().isVisible()) {
            remove(panelDescript);
            doLayout();
        }
    }

    private void pmInit() {
        CheckCellRenderer checkRenderer = null;
        TextCellRenderer  textRenderer = null;

        checkRenderer = new CheckCellRenderer();
        textRenderer = new TextCellRenderer();
        buttonListener = new LocalButtonListener();
        buttonFile.addActionListener(buttonListener);
        buttonDD.addActionListener(buttonListener);

        //
        tableDD.setModel(new DDTableModel());
        tableDD.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        tableDD.getColumnModel().getColumn(0).setCellRenderer(checkRenderer);
        tableDD.getColumnModel().getColumn(1).setCellRenderer(textRenderer);
        tableDD.getColumnModel().getColumn(2).setCellRenderer(textRenderer);
        tableDD.getColumnModel().getColumn(0).setPreferredWidth(50);
        tableDD.getColumnModel().getColumn(1).setPreferredWidth(50);
        tableDD.getColumnModel().getColumn(2).setPreferredWidth(100);
    }

    //
    private void jbInit() throws Exception {
        panelDescript =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelFile = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        textFile = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonFile = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        layoutFile =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutDescript =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonDD = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                               JButton.class.getName());
        scroll = (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                                 JScrollPane.class.getName());
        tableDD = (JTable) Beans.instantiate(getClass().getClassLoader(),
                                             JTable.class.getName());
        panelFile.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(new Color(187, 218, 252), new Color(91, 107, 123)),
                                             getTitle()));
        panelFile.setLayout(layoutFile);
        panelDescript.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(new Color(187, 218, 252), new Color(91, 107, 123)),
                                                 "Deployment Descriptors"));
        panelDescript.setLayout(layoutDescript);
        textFile.setEnabled(false);
        textFile.setText("ARCHIVE FILE");
        buttonFile.setMargin(new Insets(2, 6, 2, 6));
        buttonFile.setText("...");
        buttonDD.setMargin(new Insets(2, 6, 2, 6));
        buttonDD.setText("...");
        panelDescript.add(buttonDD,
                          new GridBagConstraints(3, 0, 1, 1, 0.1, 0.1,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(5, 0, 5, 1), 0,
                                                 0));
        panelDescript.add(scroll,
                          new GridBagConstraints(2, 0, 1, 1, 0.6, 0.6,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.BOTH,
                                                 new Insets(5, 5, 5, 5), 100,
                                                 60));
        scroll.getViewport().add(tableDD, null);
        panelFile.add(textFile,
                      new GridBagConstraints(0, 0, 1, 1, 0.6, 0.6,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 10, 2, 2), 0, 0));
        panelFile.add(buttonFile,
                      new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(1, 1, 1, 1), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelDescript,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(4, 5, 4, 5), 0, 0));
        this.add(panelFile,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(4, 5, 4, 5), 0, 0));
    }

    private void chooseArchive() {
        ExtensionFilter filter = null;
        File            choice = null;
        PathHandle      path = null;

        filter = new ExtensionFilter();
        filter.setDescriptionTitle(getTitle());
        filter.addExtension(getExtension());
        choice = SwingUtil.getFileChoice(this, getArchivePath(), filter,
                                         "Select or enter new "
                                         + getExtension() + " file name");
        if (choice == null) {
          // cancelled
        } else {
          path = PathHandle.createPathHandle(choice);
          if (path.getExtension().length() == 0) {
              path.setExtension(getExtension());
          }
          setArchivePath(path.getPath());
          if ((getWizard() != null) && path.isFile()) {
              try {
                  getWizard().readArchive(path.getPath());
              } catch (ArchiveException e) {
                  e.printStackTrace(System.err);
            }
          }
        }
    }

    private void chooseDD() {
        int            index = -1;
        DDTableModel   model = null;
        Descriptor      row = null;
        File           choice = null;
        FilenameFilter filter = null;
        PathHandle     path = null;

        //
        index = tableDD.getSelectedRow();
        if (index > -1) {
            model = (DDTableModel) tableDD.getModel();
            row = model.getDescriptor(index);
            filter = new FilenameFilter();
            filter.setIncludeName(row.getType() + ".xml");
            filter.setDescriptionTitle("Deployment Descriptor");
            if (row.getPath().trim().length() == 0
                    && getDefaultDD().length > 0) {
                path =
                    PathHandle.createPathHandle(getWorkingRoot()
                                                + File.separator
                                                + getDefaultDD()[0]);
                if (!path.isFile()) {
                    path =
                        PathHandle.createPathHandle(getWorkingRoot());
                }
            } else {
                path = PathHandle.createPathHandle(row.getPath());
            }
            choice = SwingUtil.getFileChoice(this, path.getPath(), filter,
                                             "Select deployment descriptor");
        }
        if (choice == null) {

            // no change
        } else if (choice.isFile()) {
            row.setPath(choice.getAbsolutePath());
            model.setDescriptor(index, row);
        }
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonFile) {
                chooseArchive();
            } else if (source == buttonDD) {
                chooseDD();
            }
        }

    }
    private class CheckCellRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {
            Component comp = null;
            JCheckBox check = null;

            comp = super.getTableCellRendererComponent(table, value,
                                                       isSelected, hasFocus,
                                                       row, column);
            check = new JCheckBox();
            check.setBackground(comp.getBackground());
            check.setForeground(comp.getForeground());
            if (value instanceof Boolean) {
                Boolean b = (Boolean) value;

                check.setSelected(b.booleanValue());
            }
            check.setHorizontalAlignment(SwingConstants.CENTER);
            return check;
        }

    }
    private class TextCellRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {
            Component comp = null;

            comp = super.getTableCellRendererComponent(table, value,
                                                       isSelected, false,
                                                       row, column);
            if (comp instanceof JLabel) {
                JLabel jlab = (JLabel) comp;

                if (column == 1) {
                    jlab.setHorizontalAlignment(SwingConstants.CENTER);
                } else {
                    jlab.setToolTipText(jlab.getText());
                }
            }
            return comp;
        }

    }



}
