/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package kelp.webapp.presentation;

// Xerces imports
import org.w3c.dom.html.HTMLInputElement;

// Servlet imports
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

// Standard imports
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

/**
 * <p>
 * This presentation object dynamically creates an HTML page
 * showing the greeting from the application configuration file.
 * </p>
 */
public class FormServlet extends HttpServlet {
    public void doPost(HttpServletRequest request,
                       HttpServletResponse response) throws ServletException,
                       IOException {
        doGet(request, response);
    }

    public void doGet(HttpServletRequest request,
                      HttpServletResponse response) throws ServletException,
                      IOException {
        FormHTML            page = null;
        ServletOutputStream out;
        Properties          data = null;
        byte[]              buffer;

        data = initData(request, data);
        data = saveParameters(request, data);
        page = createPage(data);
        buffer = page.toDocument().getBytes();
        response.setContentType("text/html");
        response.setContentLength(buffer.length);
        out = response.getOutputStream();
        out.write(buffer);
        out.flush();
        response.flushBuffer();
    }

    private Properties saveParameters(HttpServletRequest request,
                                      Properties data) {
        String paramFirst = null;
        String paramLast = null;

        try {
            paramFirst = request.getParameter("first");
            paramLast = request.getParameter("last");
        } catch (Exception e) {
            paramFirst = null;
            paramLast = null;
        }
        if (paramFirst != null && paramLast != null) {
            data.put("first", paramFirst);
            data.put("last", paramLast);
            writePropertyFile(data);
        }
        return data;
    }

    public FormHTML createPage(Properties data) {
        String           first = null;
        String           last = null;
        HTMLInputElement inputFirst = null;
        HTMLInputElement inputLast = null;
        FormHTML         page = null;

        page = new FormHTML();
        first = (String) data.get("first");
        last = (String) data.get("last");
        inputFirst = page.getElementFirst();
        inputLast = page.getElementLast();
        inputFirst.setValue(first);
        inputLast.setValue(last);
        return page;
    }

    private void writePropertyFile(Properties data) {
        FileOutputStream out = null;
        String           dataPath = null;

        dataPath = getDataPath();
        try {
            out = new FileOutputStream(dataPath, false);
            data.store(out, "Data for Form page");
            out.close();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
    }

    private Properties readPropertyFile(Properties data) {
        FileInputStream is = null;
        String          dataPath = null;

        dataPath = getDataPath();
        try {
            is = new FileInputStream(dataPath);
            data = new Properties();
            data.load(is);
            is.close();
        } catch (Exception e) {

            // create deault
            e.printStackTrace();
            data = new Properties();
            data.put("first", "Luke");
            data.put("last", "Skywalker");
        }
        return data;
    }

    private Properties initData(HttpServletRequest request, Properties data) {
        HttpSession session = null;

        session = request.getSession(true);

        if (session != null) {
          data = (Properties) session.getAttribute("data");
        }
        if (data == null) {
            data = readPropertyFile(data);
        }
        return data;
    }

    private String getDataPath() {
        StringBuffer pathBuf = new StringBuffer();
        String       testPath = getInitParameter("testDataPath");

        if (testPath == null) {
            System.out.println("init-parm not found: testDataPath");
        } else {
            pathBuf.append(testPath);

            // Remove quotes
            int lastAt = -1;

            lastAt = pathBuf.length() - 1;
            if (pathBuf.charAt(lastAt) == '"') {
                pathBuf.deleteCharAt(lastAt);
            }
            if (pathBuf.charAt(0) == '"') {
                pathBuf.deleteCharAt(0);
            }
            pathBuf.append('/');
            pathBuf.append("form.properties");
        }
        return pathBuf.toString();
    }

}
